package com.bull.eclipse.jonas;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */

import java.io.File;
import java.util.Vector;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;

import com.bull.eclipse.jonas.utils.TemplateDirUtil;
import com.bull.eclipse.jonas.utils.VMLauncherUtility;


/**
 * See %JONAS_ROOT/bin/nt/jonas.bat
 */	
public class Jonas41Bootstrap extends JonasBootstrap {
	
	private String startCommand = "org.objectweb.jonas.server.Server";
	private String adminCommand = "org.objectweb.jonas.adm.JonasAdmin";
	private String mainClass = "org.objectweb.jonas.server.Bootstrap";	
	/*
	 * @see JonasBootstrap#getClasspath()
	 */
	public String[] getClasspath() {
		String[] classpath = new String[4];
		classpath[0] = getJonasDir() + File.separator + "lib" + File.separator + "common" + File.separator + "ow_jonas_bootstrap.jar";
		// Add tools.jar JDK file to classpath
		String toolsJarLocation = VMLauncherUtility.getVMInstall().getInstallLocation() + File.separator + "lib" + File.separator + "tools.jar";
		classpath[1] = toolsJarLocation;
		String templateDir = TemplateDirUtil.getTemplateDir();
		classpath[2] = templateDir;
		String jonasConfig = getBaseDir() + File.separator + "conf";
		classpath[3] = jonasConfig;

		return classpath;
	}


	/*
	 * @see JonasBootstrap#getMainClass()
	 */
	public String getMainClass() {
		return mainClass;
	}

	/*
	 * @see JonasBootstrap#getStartCommand()
	 */
	public String getStartCommand() {
		return startCommand;
	}

	/*
	 * @see JonasBootstrap#getStopCommand()
	 */
	public String getStopCommand() {
		return adminCommand;

	}

	public String getAdminCommand() {
		return adminCommand;

	}

	public String[] getPrgArgs(String command,String[] options) {
		if ((command.equals(adminCommand)) && (options == null)) {	
			// Stop JOnAS	
			String[] prgArgs = new String[2];
			prgArgs[0] = adminCommand;
			prgArgs[1] = "-s";
			return prgArgs;
		} else {
			if (options != null) {
				int nbOptions = options.length;
				String [] prgArgs = new String[nbOptions + 1];
				prgArgs[0] = adminCommand;
				for (int i=0; i<nbOptions; i++) {
						JonasLauncherPlugin.log("Option " + i + " =" + options[i]);
						prgArgs[i+1] = options[i];
				}
				return prgArgs;
			} else {
				// On dmarre JOnAS
				String[] prgArgs = new String[1];
				prgArgs[0] = command;
				return prgArgs;
			}
		}				
	
		
	}


	/*
	 * @see JonasBootstrap#getVmArgs()
	 */
	public String[] getVmArgs() {
		String[] vmArgs = new String[11];
		
		vmArgs[0] = "-Dinstall.root=" + getJonasDir();
	
		vmArgs[1] = "-Djava.security.policy=" + getJonasDir() + File.separator + "conf" + File.separator + "java.policy";
		vmArgs[2] = "-Dobjectweb.orb=";		
		vmArgs[3] = "-Djonas.base=" + getBaseDir();	
		vmArgs[4] = "-Djonas.classpath=" + getXtraClasspath();		
		vmArgs[5] = "-Djonas.default.classloader=true";
		vmArgs[6] = "-Djavax.rmi.CORBA.PortableRemoteObjectClass=org.objectweb.carol.rmi.multi.MultiPRODelegate";
		vmArgs[7] = "-Djava.naming.factory.initial=org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory";
		vmArgs[8] = "-Djava.security.auth.login.config=" + getBaseDir() + File.separator + "conf" + File.separator + "jaas.config";
		vmArgs[9] = "-Djava.endorsed.dirs="  + getJonasDir() + File.separator + "lib" + File.separator + "endorsed";
		vmArgs[10] = "-Djava.rmi.server.RMIClassLoaderSpi=org.objectweb.jonas.server.RemoteClassLoaderSpi";

					
		return vmArgs;
	}

	
	/**
	 * @see JonasBootstrap#getLabel()
	 */
	public String getLabel() {
		return "Jonas 4.1";
	}

	/**
	 * @see JonasBootstrap#getJonasRMIJarPath()
	 */
	public Vector getJonasJarPath() {
		Vector vec = new Vector();
		IPath libPath = new Path("lib").append("common").append("j2ee").append("mail").append("mail.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("jts").append("jts-spec1_0.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("jta").append("jta-spec1_0_1.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("jms").append("jms.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("jacc").append("jacc-1_0.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("j2eemanagement").append("javax77.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("ejb").append("ejb-2_1-api.jar");
		vec.add(libPath);
		//libPath = new Path("lib").append("common").append("j2ee").append("ejb").append("deployment").append("deployment-1_1.jar");
		//vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("connector").append("connector-1_5.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("activation").append("activation.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("ws").append("axis").append("axis.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("ws").append("axis").append("jaxrpc.jar");		
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("ws").append("axis").append("commons-discovery.jar");		
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("ws").append("axis").append("saaj.jar");		
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("jakarta").append("commons-logging").append("commons-logging-api.jar");		
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("ow_jonas.jar");		
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("jotm").append("jotm.jar");		
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("carol").append("ow_carol_core.jar");		
		vec.add(libPath);

		return 	vec;
	}
	

	/**
	 * @see JonasBootstrap#getJonasWebJarPath()
	 */
	public Vector getJonasWebJarPath() {
		Vector vec = new Vector();
		IPath libPath = new Path("lib").append("common").append("j2ee").append("jsp").append("jsp-2_0.jar");
		vec.add(libPath);
		libPath = new Path("lib").append("common").append("j2ee").append("servlet").append("servlet-2_4.jar");		
		vec.add(libPath);
		
		return 	vec;
	}
}

