package com.bull.eclipse.jonas;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */
 
 
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.model.ISourceLocator;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.sourcelookup.JavaSourceLocator;

import com.bull.eclipse.jonas.editors.ProjectListElement;
import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.StringUtil;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;
import com.bull.eclipse.jonas.utils.VMLauncherUtility;


/**
 * Start and stop Jonas
 * Subclasses contains all information specific to a Jonas Version
 */
	
public abstract class JonasBootstrap  {

	public abstract String[] getClasspath();
	public abstract String[] getVmArgs();
	public abstract String[] getPrgArgs(String command,String[] options);
	public abstract String getStartCommand();
	public abstract String getStopCommand();
	public abstract String getAdminCommand();
	public abstract String getMainClass();
	public abstract Vector getJonasJarPath();
	public abstract Vector getJonasWebJarPath();

	abstract public String getLabel();

					
	/**
	 * See %JONAS_ROOT%/bin/nt/jonas.bat
	 */	
	public void start() throws CoreException {
		this.runJonasBootsrap(getStartCommand(), true, null);
	}
	/**
	 * See %JONAS_ROOT%/bin/nt/jonas.bat stop
	 */	
	public void stop() throws CoreException {
		this.runJonasBootsrap(getStopCommand(),  false, null);
	}

	/**
	 * See %JONAS_ROOT%/bin/nt/jonas.bat stop
	 */	
	public void admin(String[] options) throws CoreException {
		this.runJonasBootsrap(getAdminCommand(),  false, options);
	}
		
	/**
	 * Simply stop and start
	 */	
	public void restart() throws CoreException {
		JonasLauncherPlugin.getDefault().setEventRestart(true);
		this.stop();
		//The start is made in JonasPLuginClass.handleDebugEvents method
	}
	
	/**
	 * Launch a new JVM running Jonas Main class
	 * Set classpath, bootclasspath and environment variable
	 */

	private void runJonasBootsrap(String jonasBootOption, boolean showInDebugger, String[] options) throws CoreException{
		String[] prgArgs = this.getPrgArgs(jonasBootOption,options);

		String[] classpath = new String[0];
		classpath = addPreferenceJvmToClasspath(classpath);
		classpath = addPreferenceProjectListToClasspath(classpath);
		classpath = StringUtil.concat(classpath, this.getClasspath() );
				
		String[] vmArgs = this.getVmArgs();
		vmArgs = addPreferenceParameters(vmArgs);
	
		String[] bootClasspath = addPreferenceJvmToBootClasspath(new String[0]);				
		
		VMLauncherUtility.runVM(getLabel(), getMainClass(), classpath, bootClasspath, vmArgs, prgArgs, getSourceLocator(), isDebugMode(), showInDebugger);		
	}
	
	private boolean isDebugMode() {
		return JonasLauncherPlugin.getDefault().isDebugMode();
	}
	
	/**
	 * Set Jonas process source locator.
	 * Initial implementation is to return all Java Projects
	 * This could decrease debugger performance (I don't know, have to ask to eclipse team) 
	 * Todo after 2.0 is released : add sourceLocator only for allJonasProjects
	 */	
	
	private ISourceLocator getSourceLocator() throws CoreException {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IProject[] allProjects = root.getProjects();
		
		ArrayList tempList = new ArrayList(allProjects.length);
		for (int i = 0; i < allProjects.length; i++) {
			if((allProjects[i].isOpen()) && allProjects[i].hasNature(JavaCore.NATURE_ID)) {
				tempList.add(allProjects[i].getNature(JavaCore.NATURE_ID));
			}
		}		

		ISourceLocator sourceLocator = null;		
		if(!tempList.isEmpty()) {
			IJavaProject[] javaProjects = (IJavaProject[])tempList.toArray(new IJavaProject[1]);
			sourceLocator = new JavaSourceLocator(javaProjects, true);
		}
		
		return sourceLocator;
	}

	
	protected String getJonasDir() {
		return JonasLauncherPlugin.getDefault().getJonasDir();	
	}


	protected String getBaseDir() {
		return JonasLauncherPlugin.getDefault().getBaseDir();	
	}
	
	protected String getXtraClasspath() {
		return JonasLauncherPlugin.getDefault().getXtraClasspath();	
	}
	
	protected String getOrb() {
		String orb = JonasLauncherPlugin.getDefault().getOrb();
		File jndiTemplate = new File(TemplateDirUtil.getTemplateDir() + File.separator + "jndi.properties." + orb.toLowerCase());		
		File jndiFile = new File(TemplateDirUtil.getTemplateDir() + File.separator + "jndi.properties");
		try {
			FileUtil.copyFile(jndiTemplate,jndiFile);
		} catch (IOException ie) {

			//We will use the JONAS_BASE/conf/jndi.properties file
		}
		return orb;	
	}
	
	private String[] addPreferenceProjectListToClasspath(String[] previouscp) {
		List projectsList = JonasLauncherPlugin.getDefault().getProjectsInCP();
		String[] result = previouscp;
		Iterator it = projectsList.iterator();
		while(it.hasNext()) {
			try {
				ProjectListElement ple = (ProjectListElement)it.next();
				IJavaProject jproject = JavaCore.create(ple.getProject());
				result = this.addProjectToClasspath(result, jproject);	
			} catch (Exception e) {
				// nothing will be added to classpath
			}
		}
		
		return result;	
		
	}
			
	private String[] addProjectToClasspath(String[] previouscp, IJavaProject project) throws CoreException {
//		IWorkspaceRoot root= ResourcesPlugin.getWorkspace().getRoot();
//		IJavaProject project = JavaCore.getJavaCore().create(root.getProject(projectName));
		if((project != null) && (project.exists() && project.isOpen())) {
			String[] projectcp = JavaRuntime.computeDefaultRuntimeClassPath(project);
			return StringUtil.concat(projectcp, previouscp);
		} else {
			return previouscp;	
		}
	}

	private String[] addPreferenceParameters(String[] previous) {
		String[] prefParams = StringUtil.cutString(JonasLauncherPlugin.getDefault().getJvmParamaters(), JonasPluginResources.PREF_PAGE_LIST_SEPARATOR);	
		return StringUtil.concat(previous, prefParams);
	}

	private String[] addPreferenceJvmToClasspath(String[] previous) {
		String[] prefClasspath = StringUtil.cutString(JonasLauncherPlugin.getDefault().getJvmClasspath(), JonasPluginResources.PREF_PAGE_LIST_SEPARATOR);	
		return StringUtil.concat(previous, prefClasspath);
	}

	private String[] addPreferenceJvmToBootClasspath(String[] previous) {
		String[] prefBootClasspath = StringUtil.cutString(JonasLauncherPlugin.getDefault().getJvmBootClasspath(), JonasPluginResources.PREF_PAGE_LIST_SEPARATOR);	
		return StringUtil.concat(previous, prefBootClasspath);
	}


}

