package com.bull.eclipse.jonas;

/*
 * (c) Copyright Emmanuel Rias, Bull SA 2003.
 * All Rights Reserved.
 */
 
import java.util.ArrayList;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.List;

import com.bull.eclipse.jonas.editors.ClasspathFieldEditor;

/**
 * This class adds a resources list to lib Web App directory. 
 * 
 * The selection is read and written to the file ".extraresources" in the
 * root of the project.
 *  
 * @version 	1.0
 * @author		Emmanuel Rias (emmanuel.rias@bull.net)
 */

public class JonasProjectExtraResourcesPropertyPage {
	private static final String WEBAPP_EXTRA_RESOURCES_FILENAME = ".extraresources";	
	private static final String EXTRA_RESOURCES_CLASSPATH = "extraResources";
	
	private ClasspathFieldEditor erList;
	private ExtraResourcesEntries entries;
	private Composite group;
	
	private JonasProjectPropertyPage page;
	
	
	public JonasProjectExtraResourcesPropertyPage(JonasProjectPropertyPage page) {
		this.page = page;
	}
	
	public IJavaProject getJavaProject() {
		try {
			return page.getJavaProject();
		} catch (CoreException e) {
			JonasLauncherPlugin.log(e);
			return null;
		}
	}
	
	/** okay has been pressed */
	public boolean performOk() {
		List newSelection = erList.getListControl(group);
		
		try {
			page.getJonasProject().setExtraResourcesEntries(new ExtraResourcesEntries(newSelection.getItems(),newSelection.getItemCount()));
			page.getJonasProject().saveProperties();		
		} catch(Exception ex) {
			JonasLauncherPlugin.log(ex);
			return false;
		}


		return true;
	}	

	public Control getControl(Composite ctrl) {
		group   = new Composite(ctrl, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		group.setLayout(layout);
		java.util.List listExRes = readSelectedEntries();
		if (listExRes != null) 
			erList = new ClasspathFieldEditor(
						JonasLauncherPlugin.JONAS_EXTRA_RESOURCES_KEY,
						JonasPluginResources.JONAS_EXTRA_RESOURCES_LABEL,
						group,
						listExRes);
		else 
			erList = new ClasspathFieldEditor(
						JonasLauncherPlugin.JONAS_EXTRA_RESOURCES_KEY,
						JonasPluginResources.JONAS_EXTRA_RESOURCES_LABEL,
						group);
								
		erList.load();
		return group;

	}
	
	public void getExtraResourcesEntries(IJavaProject prj, ArrayList data) {
		
		IClasspathEntry[] entries = null;
		
		try {
			add(data, prj.getProject().getWorkspace().getRoot().findMember(prj.getOutputLocation()));		
			entries = prj.getResolvedClasspath(false);
		} catch(JavaModelException e) {
			JonasLauncherPlugin.log(e);
		}
		if (entries == null) return;
		for (int i = 0; i < entries.length; i++) {
			IClasspathEntry entry = entries[i];
			if (entry.getEntryKind() == IClasspathEntry.CPE_PROJECT) {
				String prjName = entry.getPath().lastSegment();
				getExtraResourcesEntries(getJavaProject().getJavaModel().getJavaProject(prjName), data);				
			} else if (entry.getEntryKind() == IClasspathEntry.CPE_LIBRARY) {
				IResource res = prj.getProject().getWorkspace().getRoot().findMember(entry.getPath());
				if (res != null) 
					add(data, res);
				else
					add(data, entry.getPath());
			} else if (entry.getEntryKind() != IClasspathEntry.CPE_SOURCE) {
				add(data, entry.getPath());
			}			
		}		
	}
	
	private void add(ArrayList data, IPath entry) {
		if (entry.isAbsolute() == false) entry = entry.makeAbsolute();
		if (!data.contains(entry.toFile().toString())) {
			data.add(entry.toFile().toString());
		}		
	}
	private void add(ArrayList data, IResource con) {
		if (con == null) return;
		add(data, con.getLocation());
	}
	
	/** reads the selected entries from persistent storage */
	private java.util.List readSelectedEntries() {
		
		java.util.List listExtraRes = null;		
		try {			
			ExtraResourcesEntries exResEntr = page.getJonasProject().getExtraResourcesEntries();
			if (exResEntr != null)
				listExtraRes = exResEntr.getList();
		} catch(Exception ex) {
			JonasLauncherPlugin.log(ex);			
		}
		return listExtraRes;
	}
	
}
