package com.bull.eclipse.jonas.actions;

import org.eclipse.core.resources.IFolder;
import org.eclipse.jdt.internal.core.PackageFragmentRoot;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bull.eclipse.jonas.JonasLauncherPlugin;

abstract public class JonasPackageRootAbstractActionDelegate implements IWorkbenchWindowActionDelegate {
	protected IWorkbenchWindow window;
	private String msg;

	/*
	 * @see IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
	}

	/*
	 * @see IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}

	/*
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {			
		Shell shell= JavaPlugin.getActiveWorkbenchShell();
		try {
			PackageFragmentRoot packageRoot = this.getCurrentSelection();	
			String actionId = action.getId();
			if((packageRoot != null) ) {
				if (this.doActionOn(packageRoot)) {			
					setMsgToSuccess();
					MessageDialog.openInformation(shell,"Jonas", msg);			
				}
			} else {
				setMsgToFail("You have to click on a folder in the resourceAdapter directory", true);
			}
		} catch (Exception ex) {		
			System.out.println("Exception : " + ex.getMessage());	
			JonasLauncherPlugin.log(ex.getStackTrace().toString());			
			setMsgToFail(ex.getMessage(), true);
			MessageDialog.openInformation(shell,"Jonas", msg);
		} 
		
	}


	protected PackageFragmentRoot getCurrentSelection() {
		IWorkbenchWindow window = JavaPlugin.getActiveWorkbenchWindow();
		PackageFragmentRoot result = null;
		if (window != null) {			
			ISelection selection= window.getSelectionService().getSelection();
			if (selection instanceof IStructuredSelection) {
				Object packageRoot = ((IStructuredSelection)selection).getFirstElement();
				if(packageRoot instanceof PackageFragmentRoot)
					result = (PackageFragmentRoot) packageRoot;	
			}						
		}
		return result;
	}
	
	abstract public boolean doActionOn(PackageFragmentRoot packageRoot) throws Exception;

	/**
	 * Sets the msg.
	 * @param msg The msg to set
	 */
	private void setMsgToFail(String detail, boolean seelog) {
		this.msg = JonasLauncherPlugin.getResourceString("msg.action.failed");
		this.msg += "\n" + detail;
		if(seelog) {	
			this.msg += JonasLauncherPlugin.getResourceString("msg.action.seelog"); 
		}
	}
	
	/**
	 * Sets the msg.
	 * @param msg The msg to set
	 */
	private void setMsgToSuccess() {
		this.msg = JonasLauncherPlugin.getResourceString("msg.action.succeeded");
	}
	
	/*
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {

	}
	
}

