package com.bull.eclipse.jonas.actions.hibernate;

import java.beans.Introspector;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IBuffer;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IImportDeclaration;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaModel;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageDeclaration;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.internal.core.JavaProject;
import org.eclipse.jdt.internal.core.PackageFragment;
import org.eclipse.jdt.internal.core.PackageFragmentRoot;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceContentProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.CheckedTreeSelectionDialog;
import org.eclipse.ui.dialogs.ListSelectionDialog;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasPluginResources;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.actions.JonasFolderAbstractActionDelegate;
import com.bull.eclipse.jonas.actions.JonasPackageAbstractActionDelegate;
import com.bull.eclipse.jonas.actions.JonasPackageRootAbstractActionDelegate;
import com.bull.eclipse.jonas.actions.persistence.utils.PersistencePackage;
import com.bull.eclipse.jonas.utils.AntUtils;
import com.bull.eclipse.jonas.utils.DBToJavaNameTransformUtil;
import com.bull.eclipse.jonas.utils.EjbManagement;
import com.bull.eclipse.jonas.utils.UpdatePropsFile;
import com.bull.eclipse.jonas.utils.xml.FileXmlUtils;
import com.bull.eclipse.jonas.utils.xml.UpdateXmlFile;
import com.bull.eclipse.jonas.utils.xml.XMLUtils;
import com.bull.eclipse.newbean.FacadeEJBGenerate;
import com.bull.eclipse.newbean.EntityDescValue;
import com.bull.eclipse.newbean.FacadeValue;
import com.bull.eclipse.newbean.FkTableAndColumns;
import com.bull.eclipse.newbean.MethodDescValue;
import com.bull.eclipse.newbean.NewFacadeValue;
import com.bull.eclipse.persitence.HibernateGlobalCFGGenerate;
import com.bull.eclipse.persitence.HibernateServicePersistenceGenerate;



public class BuildHibernatePersistenceActionDelegate extends JonasPackageRootAbstractActionDelegate
{
	static private Object[] oldSelection = null; 
	static private String oldBeanName = null;
	
	public boolean doActionOn(PackageFragmentRoot packageRoot) throws Exception {
		// Update build.properties with JONAS_ROOT
		JonasProject jp = (JonasProject)packageRoot.getJavaProject().getProject().getNature(JonasLauncherPlugin.NATURE_ID);
		UpdatePropsFile.updatePropsProject(jp);		
		
		
		ArrayList hbmList = new ArrayList();
		
		String projDir = jp.getProject().getLocation().toOSString();
		String packagePersistence = PersistencePackage.getPersistencePackage(packageRoot.getChildren());
		IJavaElement res[] = packageRoot.getChildren();
		for(int i=0;i<res.length;i++) {
			IJavaElement ele = (IJavaElement)res[i];
			Object[] nonJavaFile = packageRoot.getPackageFragment(ele.getElementName()).getNonJavaResources();
			if (nonJavaFile.length != 0) {
				for (int j=0;j<nonJavaFile.length;j++) {
					IResource resource = (IResource) nonJavaFile[j];
					if (resource.getName().endsWith(".hbm.xml")) {
						hbmList.add(ele.getElementName().replace('.','/') + "/" + resource.getName());
					}					
				}
			}
		}
		
		// CFG file generation
		GlobalHBMValue ghv = new GlobalHBMValue();
		IPreferenceStore pref =	JonasLauncherPlugin.getDefault().getPreferenceStore();
		ghv.setDatasourceJNDI(pref.getString("datasource"));
		ghv.setDialect("net.sf.hibernate.dialect.DB2Dialect");
		ghv.setHbmList(hbmList);
		
		HibernateGlobalCFGGenerate hghg = new HibernateGlobalCFGGenerate(jp);
		hghg.generateCFG(ghv);
		
		//End of CFG file generation
		
		//Generate the persistence service
		PersistenceServicesValue psv = new PersistenceServicesValue();
		psv.setPersistencePackage(packagePersistence);
		psv.setPersistenceName("Persistence");
		ArrayList valueObjects = new ArrayList();
		// It is not the best thing to re start the loop but it is more clear !!
		for(int i=0;i<res.length;i++) {
			IJavaElement ele = (IJavaElement)res[i];
			Object[] nonJavaFile = packageRoot.getPackageFragment(ele.getElementName()).getNonJavaResources();
			if (nonJavaFile.length != 0) {
				ValueObjectDescription vod = new ValueObjectDescription();
				for (int j=0;j<nonJavaFile.length;j++) {
					IResource resource = (IResource) nonJavaFile[j];
					File hbmFile = new File(resource.getLocation().toOSString());
					String classValue = (String) FileXmlUtils.getAttributeValuesOfNode(hbmFile, "class", "name").iterator().next(); //We have only one occurence
					vod.setPackageName(classValue.substring(0,classValue.lastIndexOf(".")));
					vod.setValueName(classValue.substring(classValue.lastIndexOf(".")+1,classValue.length()));
					vod.setValueTOName(vod.getValueName().substring(0,vod.getValueName().length()-5) + "TO");
					String tableValue = (String) FileXmlUtils.getAttributeValuesOfNode(hbmFile, "class", "table").iterator().next();//We have only one occurence
					vod.setObjectName(DBToJavaNameTransformUtil.dbNameToVariableName(tableValue));
					String pkType = null;
					Collection pkTypes = FileXmlUtils.getAttributeValuesOfNode(hbmFile, "id", "type"); //We have only one occurence
					if (pkTypes.size() == 0) {
						//Composite PK
						pkType = (String) FileXmlUtils.getAttributeValuesOfNode(hbmFile, "composite-id", "class").iterator().next(); //We have only one occurence
						ICompilationUnit icompil = packageRoot.getPackageFragment(ele.getElementName()).getCompilationUnit(vod.getObjectName() + "PK.java");
						IType type = icompil.getTypes()[0];
						IField field[] = type.getFields();
						CompositePKValue pkValue = new CompositePKValue();
						ArrayList alPk = new ArrayList();
						for (int f=0;f<field.length;f++) {
							pkValue = new CompositePKValue();
							pkValue.setJavaName(field[f].getElementName());
							pkValue.setColumnName(field[f].getElementName());
							alPk.add(pkValue);
						}
						HibernateServicePersistenceGenerate hspg1 = new HibernateServicePersistenceGenerate(jp);
						hspg1.generateCompositePK(alPk, vod.getPackageName(), vod.getObjectName() + "PK");
												
						majHbmFileWithCompositePK(hbmFile,jp);
						
					} else {
						pkType = (String) pkTypes.iterator().next();
					}
					vod.setPkJavaType(pkType);
					Collection manyRelationsName = FileXmlUtils.getAttributeValuesOfNode(hbmFile, "set", "name");
					if (manyRelationsName != null) {
						ArrayList manyRelationNameCapitalise = new ArrayList();
						Iterator ite = manyRelationsName.iterator(); 
						while (ite.hasNext()) {
							manyRelationNameCapitalise.add(DBToJavaNameTransformUtil.capitalise((String)ite.next()));
						}
						Collection manyRelationsObject = FileXmlUtils.getAttributeValuesOfNode(hbmFile, "one-to-many", "class");
						ArrayList manyRelationObjectValue = new ArrayList();
						ite = manyRelationsObject.iterator();
						while (ite.hasNext()) {
							String manyRelationObject = (String)ite.next();
							manyRelationObjectValue.add(manyRelationObject);
						}
						
						// Many relation information save in the value object
						if (manyRelationNameCapitalise.size() == manyRelationObjectValue.size()) {
							ArrayList manyRelationObject = new ArrayList();
							for(int u=0;u<manyRelationNameCapitalise.size();u++) {														
								ValueObjectDescription vodMany = new ValueObjectDescription();
								
								vodMany.setPackageName(((String)manyRelationObjectValue.get(u)).substring(0,((String)manyRelationObjectValue.get(u)).lastIndexOf(".")));
								vodMany.setValueName(((String)manyRelationObjectValue.get(u)).substring(((String)manyRelationObjectValue.get(u)).lastIndexOf(".")+1,((String)manyRelationObjectValue.get(u)).length()));
								vodMany.setValueTOName(vodMany.getValueName().substring(0,vodMany.getValueName().length()-5) + "TO");
								vodMany.setObjectName((String)manyRelationNameCapitalise.get(u));
								if (vodMany.getObjectName().indexOf("By") != -1) {
									vodMany.setSuffix(vodMany.getObjectName().substring(vodMany.getObjectName().indexOf("By")));
								} else
									vodMany.setSuffix("");
								
								manyRelationObject.add(vodMany);
							}
							vod.setRelationMany(manyRelationObject);
						} else {
							//Problems !!!!!							
							throw new Exception("There is a mysterious problem !!");
						}
					}
					Collection oneRelationsName = FileXmlUtils.getAttributeValuesOfNode(hbmFile, "many-to-one", "name");
					Collection oneRelationsClass = FileXmlUtils.getAttributeValuesOfNode(hbmFile, "many-to-one", "class");
					if (oneRelationsName != null) {
						ArrayList oneRelationNameCapitalise = new ArrayList();
						Iterator iteName = oneRelationsName.iterator();
						Iterator iteClass = oneRelationsClass.iterator();
						// Many relation information save in the value object
						if (oneRelationsName.size() == oneRelationsClass.size()) {
							ArrayList oneRelationObject = new ArrayList();
							while (iteName.hasNext()) {														
								ValueObjectDescription vodOne = new ValueObjectDescription();
								String name = (String) iteName.next();
								String value = (String) iteClass.next();
								vodOne.setPackageName(value.substring(0,value.lastIndexOf(".")));
								vodOne.setValueName(value.substring(value.lastIndexOf(".")+1,value.length()));
								vodOne.setValueTOName(vodOne.getValueName().substring(0,vodOne.getValueName().length()-5) + "TO");
								vodOne.setObjectName(DBToJavaNameTransformUtil.capitalise(name));
								if (vodOne.getObjectName().indexOf("By") != -1) {
									vodOne.setSuffix(vodOne.getObjectName().substring(vodOne.getObjectName().indexOf("By")));
								} else
									vodOne.setSuffix("");
								
								oneRelationObject.add(vodOne);
							}
							vod.setRelationOne(oneRelationObject);
						} else {
							//Problems !!!!!							
							throw new Exception("There is a mysterious problem !!");
						}
					}
				}			
				valueObjects.add(vod);
			}
		}
		psv.setPackagesOfValueObject(valueObjects);
		
		HibernateServicePersistenceGenerate hspg = new HibernateServicePersistenceGenerate(jp);
		hspg.generateServicePersistence(psv);
		
		jp.getProject().refreshLocal( IResource.DEPTH_INFINITE, null );			 
		
		
		
	    return false;
	}			


	public void majHbmFileWithCompositePK(File hbmFile, JonasProject jp) {
		UpdateXmlFile.updateHbmXmlFile(hbmFile, jp.getProject().getLocation().append("temp").append("composite_id.xml").toFile());
	}
}

