/*
 * Created on Sep 22, 2003
 *
 */
package com.bull.eclipse.jonas.actions.ws;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;

/**
 * Interface with the JOnAS 'newbean' functionality.
 * @author steve krupanszky
 *
 */
public class CreateDeployWSDD
{
	//private JonasProject	jonasNature;
	private IFolder			pkgfolder;	// the folder in which the java files will go
	private JonasProject		project;
	private VelocityEngine vEngine = null;
	private VelocityContext vContext = null;
	
	private static final int EXIT_SUCCESS = 0;
	private static final int EXIT_FAILURE = 1;


	public			CreateDeployWSDD( JonasProject proj)
	{
		project = proj;
	}

	/**
	 * Copy file into web directory, if it doesn't already exist there.
	 */
	private void	copyFileToWebAppDirectory( File f )
	{
		IFile	xmlfile = project.getProject().getFile( f.getName() );
		try{ FileUtil.copy( f, xmlfile.getLocation().toFile() ); }
		catch( IOException e ) {
			JonasLauncherPlugin.log(
				"JOPE: exception copying file " + f.getName()
				+ " to project " + project.getProject().getName() + ": " + e );
		}
	}

	private boolean		myDelete( File f )
	{
		boolean		b = f.delete();
		int			retryCount = 5;
		while( !b && retryCount-->0 ) {
			try{ java.lang.Thread.sleep(1000); } catch(Exception e) {}
			b = f.delete();
		}
		return b;
	}

	public void	runUpdate(String endpointName, String jndiName, String homeClass, 
					String remoteClass, String homeLocalClass, String localClass, String packName)
	{
		vContext = new VelocityContext();

		String jonasRoot = JonasLauncherPlugin.getDefault().getJonasDir();
		
		vContext.put("endpoint",endpointName);		
					
		vContext.put("jndiName", jndiName);
		vContext.put("localHomeClassName", homeLocalClass);
		vContext.put("localClassName", localClass);
		vContext.put("homeClassName", homeClass);
		vContext.put("remoteClassName", remoteClass);
		JonasLauncherPlugin.log("Deploy = " + project.getProject().getLocation().append(project.getSrcEJBDir() + File.separator + packName ));
		File outputDir = new File(project.getProject().getLocation().append(project.getSrcEJBDir() + File.separator + packName ).toOSString());
		

		// Generates webapps files
		try {
			vEngine = new VelocityEngine();
			vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
			vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
			vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
							TemplateDirUtil.getTemplateDir().concat("/ws"));
			try {
				vEngine.init();
			} catch (Exception e) {
				fatalError("unable to initilise Velocity engine (" + e + ")");
			}
			generate("deploy.wsdd", outputDir + "/deploy.wsdd");            			
		} catch (Exception e) {        	
			error(e.toString());
		}
		// now to refresh project so it picks up the new files
		try{ project.getProject().refreshLocal( IResource.DEPTH_INFINITE, null ); }
		catch( CoreException e ) {}
				
	}

	public void			setPkgFolder( IFolder f )
	{
		pkgfolder = f;
	}

	/**
	 * Create a new array : [ s1, s2, s3 ]
	 * (TODO move this method to StringUtil)
	 */
	public static String[] concat( String[] s1, String s2, String s3 )
	{
		String[] full = new String[ s1.length + 2 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-2 ] = s2;
		full[ full.length-1 ] = s3;
		return full;	
	}

	/**
	 * Create a new array : [ s1, s2 ]
	 * (TODO move this method to StringUtil)
	 */
	public static String[] concat( String[] s1, String s2 )
	{
		String[] full = new String[ s1.length + 1 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-1 ] = s2;
		return full;	
	}

	static void fatalError(String errMsg) {
		System.err.println("NewBean fatal error: " + errMsg);
		System.exit(EXIT_FAILURE);
	}

	/**
	  * Generates a file from the specified template.
	  * @param templateFileName the name of the template file
	  * @param targetFileName the name of the generated file
	  */
	 private void generate(String templateFileName,
						   String targetFileName) throws Exception, IOException, ResourceNotFoundException, ParseErrorException, MethodInvocationException {
		 FileWriter fileWriter = null;
		 fileWriter = new FileWriter(targetFileName);
		 vEngine.mergeTemplate(templateFileName, vContext, fileWriter);
		 fileWriter.close();
	 }

	/**
	 * Display the specified error message.
	 * @param errMsg the error message to display
	 */
	static void error(String errMsg) {
		System.err.println("NewBean error: " + errMsg);
	}


}
