package com.bull.eclipse.jonas.editors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.CheckedListDialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.DialogField;
import org.eclipse.jdt.internal.ui.wizards.dialogfields.LayoutUtil;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.ide.IDE.SharedImages;

import com.bull.eclipse.jonas.JonasPluginResources;

public class ProjectListEditor implements JonasPluginResources {
			
	private CheckedListDialogField fProjectsList;
	
	public ProjectListEditor() {
		
		String[] buttonLabels= new String[] {
			PREF_PAGE_SELECTALL_LABEL, 
			PREF_PAGE_UNSELECTALL_LABEL
		};
		
		fProjectsList= new CheckedListDialogField(null, buttonLabels, new MyLabelProvider());
		fProjectsList.setLabelText(PREF_PAGE_PROJECTINCP_LABEL);
		fProjectsList.setCheckAllButtonIndex(0);
		fProjectsList.setUncheckAllButtonIndex(1);
		updateProjectsList();		
//		fProjectsList.setViewerSorter(new CPListElementSorter());
	}
	
	public void init(IJavaProject jproject) {
		updateProjectsList();
	}
		
	private void updateProjectsList() {
		try {
			IWorkspaceRoot root= ResourcesPlugin.getWorkspace().getRoot();
			IProject[] projects = root.getProjects();
			
			List projectsList = new ArrayList(projects.length);
		
			for (int i= 0; i < projects.length; i++) {
				IProject proj= projects[i];
				if(projects[i].isOpen()) {
					projectsList.add(new ProjectListElement(proj));
				}
			}	
								
			fProjectsList.setElements(projectsList);			
				
		} catch (Exception e) {
			fProjectsList.setElements(new ArrayList(5));
		}

	}		
		
	// -------- UI creation ---------
		
	public Control getControl(Composite parent) {
		Composite composite= new Composite(parent, SWT.NONE);

//		fProjectsList.doFillIntoGrid(composite,3);			
		LayoutUtil.doDefaultLayout(composite, new DialogField[] { fProjectsList }, true, 0, 0);
				
		return composite;
	}
		
	public List getCheckedElements() {
		return fProjectsList.getCheckedElements();
	}
	
	public void setCheckedElements(List projects) {
		fProjectsList.setCheckedElements(projects);
	}	  
	
	private class MyLabelProvider extends LabelProvider {
		
		/*
		 * @see ILabelProvider#getImage(Object)
		 */
		public Image getImage(Object element) {
			IWorkbench workbench= JavaPlugin.getDefault().getWorkbench();
			return workbench.getSharedImages().getImage(SharedImages.IMG_OBJ_PROJECT);
		}

		/*
		 * @see ILabelProvider#getText(Object)
		 */
		public String getText(Object element) {
			return super.getText(element);
		}

	}

}
