package com.bull.eclipse.jonas.preferencepages;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.internal.debug.ui.launcher.ComboFieldEditor;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasPluginResources;
import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;
import com.bull.eclipse.jonas.utils.Zipper;


public class JonasDBMPreferencePage extends PreferencePage implements IWorkbenchPreferencePage, JonasPluginResources {

	static final private int FIELD_WIDTH = 50;
	static final private String[][] mapperListe = new String[7][2];
	static final private String[][] urldriverListe = new String[7][2];

	private static final int TEXT_FIELD_WIDTH = 200;

	private ComboFieldEditor databaseChoice;
	private StringFieldEditor urlDBM;
	private StringFieldEditor driverDBM;
	private StringFieldEditor user;
	private StringFieldEditor password;
	private StringFieldEditor datasource;
	private StringFieldEditor rdbmapper;
	private StringFieldEditor schema;
	private StringFieldEditor catalog;
	private FileFieldEditor jdbcDriver;
	private BooleanFieldEditor automaticDeploy;
	
	private VelocityEngine vEngine = null;
	private VelocityContext vContext = null;
	private static final int EXIT_SUCCESS = 0;
	private static final int EXIT_FAILURE = 1;


	
	public JonasDBMPreferencePage() {
		super();
		setPreferenceStore(JonasLauncherPlugin.getDefault().getPreferenceStore());
		
		mapperListe[0][0] = "DB2";
		mapperListe[0][1] = "rdb.db2";
		mapperListe[1][0] = "Oracle 9";
		mapperListe[1][1] = "rdb.oracle";
		mapperListe[2][0] = "Default";
		mapperListe[2][1] = "rdb";
		mapperListe[3][0] = "McKoi";
		mapperListe[3][1] = "rdb.mckoi";
		mapperListe[4][0] = "Postgres";
		mapperListe[4][1] = "rdb.postgres";
		mapperListe[5][0] = "Oracle 8";
		mapperListe[5][1] = "rdb.oracle8";
		mapperListe[6][0] = "MySQL";
		mapperListe[6][1] = "rdb.mysql";
		

		urldriverListe[0][0] = "url:db2:";
		urldriverListe[0][1] = "driver DB2";
		urldriverListe[1][0] = "jdbc:oracle:thin:@<your-hostname>:1521:<your-db>";
		urldriverListe[1][1] = "oracle.jdbc.driver.OracleDriver";
		urldriverListe[2][0] = "jdbc:url";
		urldriverListe[2][1] = "Driver";
		urldriverListe[3][0] = "jdbc:mckoi://localhost";
		urldriverListe[3][1] = "com.mckoi.JDBCDriver";
		urldriverListe[4][0] = "jdbc:postgresql://localhost:5432/db_jonas";
		urldriverListe[4][1] = "org.postgresql.Driver";
		urldriverListe[5][0] = "jdbc:oracle:thin:@<your-hostname>:1521:<your-db>";
		urldriverListe[5][1] = "oracle.jdbc.driver.OracleDriver";
		urldriverListe[6][0] = "jdbc:mysql://<your-hostname>/<your-db>";
		urldriverListe[6][1] = "com.mysql.jdbc.Driver";

		
//		setDescription("");
	}

	/*
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout gl = new GridLayout();
		gl.numColumns = 1;
		composite.setLayout(gl);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.grabExcessVerticalSpace = true;
		gd.grabExcessHorizontalSpace = true;
		composite.setLayoutData(gd);
				
		Composite group1 = new Composite(composite,SWT.NONE);
		GridLayout gl1 = new GridLayout(3,false);
		group1.setLayout(gl1);
		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		group1.setLayoutData(gd);		

		Composite group2 = new Composite(composite,SWT.NONE);
		GridLayout gl2 = new GridLayout(3,false);
		group1.setLayout(gl2);
		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		group2.setLayoutData(gd);		

		Composite group3 = new Composite(composite,SWT.NONE);
		GridLayout gl3 = new GridLayout(3,false);
		group1.setLayout(gl3);
		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		group3.setLayoutData(gd);		

		databaseChoice = new ComboFieldEditor(
				JonasLauncherPlugin.JONAS_PREF_DATABASE_KEY,
				PREF_PAGE_DATABASE_LABEL,
				mapperListe,
				group1);


		urlDBM = new StringFieldEditor("url",URL_DBM_LABEL,60,group1);
		driverDBM = new StringFieldEditor("driver",DRIVER_DBM_LABEL,60,group1);
		user = new StringFieldEditor("user",USER_DBM_LABEL,60,group1);
		password = new StringFieldEditor("password",PASSWORD_DBM_LABEL,60,group1);
		datasource = new StringFieldEditor("datasource",DATASOURCE_DBM_LABEL,60,group1);
		rdbmapper = new StringFieldEditor("rdbmapper",RDBMAPPER_DBM_LABEL,60,group1);
		schema = new StringFieldEditor("schema",SCHEMA_DBM_LABEL,60,group1);
		catalog = new StringFieldEditor("catalog",CATALOG_DBM_LABEL,60,group1);
		jdbcDriver = new FileFieldEditor("jdbcDriver",
								JDBC_DBM_LABEL,
								group2);

		automaticDeploy = new BooleanFieldEditor(
				JonasLauncherPlugin.JONAS_PREF_AUTOMATICDEPLOY_KEY,
				PREF_PAGE_AUTOMATICDEPLOY_LABEL,
				group3);

		databaseChoice.setPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				String newValue = (String)event.getNewValue();
				for(int i = 0;i<mapperListe.length;i++) {
					if (newValue.equals(mapperListe[i][1])) {
						rdbmapper.setStringValue(newValue);
						urlDBM.setStringValue(urldriverListe[i][0]);
						driverDBM.setStringValue(urldriverListe[i][1]);
						break;
					}										
				}				 
				}				
			}			
		);
		
		this.initField(databaseChoice);
		this.initField(urlDBM);
		this.initField(driverDBM);
		this.initField(user);
		this.initField(password);
		this.initField(datasource);
		this.initField(rdbmapper);
		this.initField(schema);
		this.initField(catalog);
		this.initField(jdbcDriver);
		this.initField(automaticDeploy);
		
		return composite;
	}

	/*
	 * @see IWorkbenchPreferencePage#init(IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}


	public boolean performOk() {
		databaseChoice.store();
		urlDBM.store();
		driverDBM.store();
		user.store();
		password.store();
		datasource.store();	
		rdbmapper.store();
		schema.store();
		catalog.store();
		jdbcDriver.store();
		automaticDeploy.store();
		
		JonasLauncherPlugin.getDefault().savePluginPreferences();
		if ((datasource.getStringValue() != null) && (!datasource.getStringValue().equals("")))
			runNewds();
		// Add the RA jdbc rar file in JONAS_BASE autoload
			
		return true;	
	}
	
	private void initField(FieldEditor field) {
		field.setPreferenceStore(getPreferenceStore());
		field.setPreferencePage(this);
		field.load();		
	}



	private void runNewds()
	{
		String jonasRoot = JonasLauncherPlugin.getDefault().getJonasDir();
		String tmp = System.getProperty("java.io.tmpdir");
		SimpleDateFormat	df = new SimpleDateFormat("yyyyMMdd_HHmmssSSS");
		StringBuffer	datim = df.format( new Date(), new StringBuffer(), new FieldPosition(0) );
		File	tmpdir = new File( tmp + "/JOPE_" + datim);
		boolean iscreate = tmpdir.mkdir();
		if( !iscreate ) {
			JonasLauncherPlugin.log("JOPE: unable to create temp dir: "+tmpdir);
			return;
		}
		File	tmpdirMETA = new File(tmpdir,"META-INF");
		iscreate = tmpdirMETA.mkdir();
		if( !iscreate ) {
			JonasLauncherPlugin.log("JOPE: unable to create temp META dir: "+tmpdir);
			return;
		}

		vContext = new VelocityContext();

		vContext.put("url", urlDBM.getStringValue() );
		vContext.put("datasource", datasource.getStringValue() );
		vContext.put("driver", driverDBM.getStringValue() );
		vContext.put("user", user.getStringValue() );
		vContext.put("password", password.getStringValue() );
		vContext.put("mapper", rdbmapper.getStringValue() );
		
        vEngine = new VelocityEngine();
        vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
        vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
		vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
				TemplateDirUtil.getTemplateDir());

		try {
			vEngine.init();
		} catch (Exception e) {
			fatalError("unable to initilise Velocity engine (" + e + ")");
		}
		
        // Generates bean files
        try {
            generate("jonas-ra.vm", tmpdirMETA.getAbsolutePath() + "/jonas-ra.xml");
        } catch (Exception e) {
            error(e.toString());
        }

		
		try {
			File jarFile = new File(tmpdir,"ow_jonas_jdbc_ra.jar");
			File raFile = new File(tmpdirMETA, "ra.xml");
			File jdbcdriverFile = null;
			JonasLauncherPlugin.log("jdbcDriver = " + jdbcDriver.getStringValue());
			if ((jdbcDriver.getStringValue() != null) && (!jdbcDriver.getStringValue().equals("")))
				jdbcdriverFile = new File(jdbcDriver.getStringValue());
			File jdbcdriver = new File(tmpdir,"jdbc_driver.jar");
			FileUtil.copy(new File(JonasLauncherPlugin.getDefault().getJonasDir().concat("/lib/common/ow_jonas_jdbc_ra.jar")), jarFile);
			FileUtil.copy(new File(TemplateDirUtil.getTemplateDir().concat("/ra.xml")), raFile);
			if (jdbcdriverFile != null)
				FileUtil.copy(jdbcdriverFile, jdbcdriver);
			String ds = datasource.getStringValue().replaceAll("/","");
			Zipper zipRar = new Zipper(new File(JonasLauncherPlugin.getDefault().getBaseDir().concat("/rars/JOPE_" + ds + ".rar")),tmpdir);
			zipRar.zip();
			jarFile.delete();
			raFile.delete();
			jdbcdriver.delete();
			(new File(tmpdirMETA,"jonas-ra.xml")).delete();
			
			// automatic deployment
			if (automaticDeploy.getBooleanValue()) {
				try {
					String[] options = {"-a", "JOPE_" + datasource.getStringValue() + ".rar"};	
					JonasLauncherPlugin.getDefault().getJonasBootstrap().admin(options);
				} catch (CoreException ce) {
					JonasLauncherPlugin.log("DS deploy problem  "); 
					ce.printStackTrace();
				}
			}
		} catch (IOException ioe) {
			JonasLauncherPlugin.log( "JOPE: " + ioe.toString() );			
		}
		
	}

    /**
     * Generates a file from the specified template.
     * @param templateFileName the name of the template file
     * @param targetFileName the name of the generated file
     */
    private void generate(String templateFileName,
                          String targetFileName) throws Exception, IOException, ResourceNotFoundException, ParseErrorException, MethodInvocationException {
        FileWriter fileWriter = null;
        fileWriter = new FileWriter(targetFileName);
        vEngine.mergeTemplate(templateFileName, vContext, fileWriter);
        fileWriter.close();
    }


	/**
	 * Create a new array : [ s1, s2, s3 ]
	 * (TODO move this method to StringUtil)
	 */
	private static String[] concat( String[] s1, String s2, String s3 )
	{
		String[] full = new String[ s1.length + 2 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-2 ] = s2;
		full[ full.length-1 ] = s3;
		return full;	
	}

	/**
	 * Create a new array : [ s1, s2 ]
	 * (TODO move this method to StringUtil)
	 */
	private static String[] concat( String[] s1, String s2 )
	{
		String[] full = new String[ s1.length + 1 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-1 ] = s2;
		return full;	
	}

    /**
     * Display the specified error message.
     * @param errMsg the error message to display
     */
    static void error(String errMsg) {
        System.err.println("NewBean error: " + errMsg);
    }


    /**
     * Display the specified error message and exits with an
     * EXIT_FAILURE status.
     * @param errMsg the error message to display
     */
    static void fatalError(String errMsg) {
        System.err.println("NewBean fatal error: " + errMsg);
        System.exit(EXIT_FAILURE);
    }

	
}

