package com.bull.eclipse.jonas.preferencepages;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */

import java.io.File;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.DirectoryFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasPluginResources;
import com.bull.eclipse.jonas.editors.ClasspathFieldEditor;


public class JonasPreferencePage extends PreferencePage implements IWorkbenchPreferencePage, JonasPluginResources {

	static final private int FIELD_WIDTH = 50;
	private RadioGroupFieldEditor version;
	private DirectoryFieldEditor home;
	private DirectoryFieldEditor base;
	private RadioGroupFieldEditor orb;
	private ClasspathFieldEditor xtraClasspath;
	
	private BooleanFieldEditor debugModeEditor;
	private ModifyListener fModifyListener;
		
	public JonasPreferencePage() {
		super();
		setPreferenceStore(JonasLauncherPlugin.getDefault().getPreferenceStore());
//		setDescription("");

	}

	/*
	 * @see PreferencePage#createContents(Composite)
	 */

	protected Control createContents(Composite parent) {


		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());

		Composite versionGroup = new Composite(composite,SWT.NONE);
		
		version = new RadioGroupFieldEditor(
				JonasLauncherPlugin.JONAS_PREF_VERSION_KEY,
				PREF_PAGE_CHOOSEVERSION_LABEL,
				1,
				new String[][] {
					{PREF_PAGE_VERSION30_LABEL, JonasLauncherPlugin.JONAS_VERSION30},
					{PREF_PAGE_VERSION31_LABEL, JonasLauncherPlugin.JONAS_VERSION31},
					{PREF_PAGE_VERSION40_LABEL, JonasLauncherPlugin.JONAS_VERSION40},
					{PREF_PAGE_VERSION41_LABEL, JonasLauncherPlugin.JONAS_VERSION41},
					{PREF_PAGE_VERSION414_LABEL, JonasLauncherPlugin.JONAS_VERSION414},
				},
				versionGroup);
				
		new Label(composite, SWT.NULL); //blank

						
		Composite homeGroup = new Composite(composite,SWT.NONE);
		GridData gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		homeGroup.setLayoutData(gd);		
		home = new DirectoryFieldEditor(
				JonasLauncherPlugin.JONAS_PREF_HOME_KEY,
				PREF_PAGE_HOME_LABEL,
				homeGroup);
				
		base = new DirectoryFieldEditor(
				JonasLauncherPlugin.JONAS_PREF_BASE_KEY,
				PREF_PAGE_BASE_LABEL,
				homeGroup);
					
		home.setPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				if(event.getProperty().equals(FieldEditor.VALUE)) {		
					computeConfigFile();
				}
			}
		});


	
		new Label(composite, SWT.NULL); //blank

		final Composite orbGroup = new Composite(composite,SWT.NONE);
		
		orb = new RadioGroupFieldEditor(
						JonasLauncherPlugin.JONAS_PREF_ORB_KEY,
						PREF_PAGE_ORB_LABEL,
						1,
						new String[][] {
							{PREF_PAGE_RMI_LABEL, JonasLauncherPlugin.JONAS_ORB_RMI},
							{PREF_PAGE_JEREMIE_LABEL, JonasLauncherPlugin.JONAS_ORB_JEREMIE},
						},
						orbGroup);
		
		
		version.setPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				String newValue = (String)event.getNewValue();
				if (!newValue.equals(event.getOldValue())) {
					if (!newValue.equals(JonasLauncherPlugin.JONAS_VERSION30)) {
						orb.setEnabled(false,orbGroup);		
					} else {
						orb.setEnabled(true,orbGroup);		
					}
				}				
			}
		});
			
		new Label(composite, SWT.NULL); //blank										
		
		xtraClasspath = new ClasspathFieldEditor(
						JonasLauncherPlugin.JONAS_PREF_XTRA_CLASSPATH_KEY,
						PREF_PAGE_XTRACLASSPATH_LABEL,
						composite);

		new Label(composite, SWT.NULL); 


		this.initField(version);				
		this.initField(home);
		this.initField(base);
		this.initField(xtraClasspath);
		this.initField(orb);
		
		if (!version.getPreferenceStore().getString(JonasLauncherPlugin.JONAS_PREF_VERSION_KEY).equals(JonasLauncherPlugin.JONAS_VERSION30))
			orb.setEnabled(false,orbGroup);
		
		if(base.getStringValue().length() == 0) {
			computeConfigFile();	
		}
		
		new Label(composite, SWT.NULL); //blank

		Composite debugModeGroup = new Composite(composite,SWT.NONE);
		debugModeEditor = new BooleanFieldEditor(
			JonasLauncherPlugin.JONAS_PREF_DEBUGMODE_KEY,
			PREF_PAGE_DEBUGMODE_LABEL,
			debugModeGroup);
		if (File.separator.equals("/")) {	
			debugModeEditor.setEnabled(false,debugModeGroup);
		}
		this.initField(debugModeEditor);

		return composite;
	}

	/*
	 * @see IWorkbenchPreferencePage#init(IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}


	public boolean performOk() {	
		version.store();
		home.store();		
		JonasLauncherPlugin.getDefault().initJonasClasspathVariable();
		base.store();
		debugModeEditor.store();
		orb.store();
		xtraClasspath.store();

		JonasLauncherPlugin.getDefault().savePluginPreferences();

		return true;	
	}
	
	private void initField(FieldEditor field) {		
		field.setPreferenceStore(getPreferenceStore());
		field.setPreferencePage(this);
		field.load();		
	}
	
	private void computeConfigFile() {
		base.setStringValue(home.getStringValue());	
	}
	
}

