package com.bull.eclipse.jonas.utils;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */


import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.ISourceLocator;
import org.eclipse.debug.ui.DebugUITools;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.IVMInstallType;
import org.eclipse.jdt.launching.IVMRunner;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jdt.launching.VMRunnerConfiguration;

import com.bull.eclipse.jonas.JonasLauncherPlugin;




/**
 * Utility class for launching a JVM in Eclipse and registering it to debugger
 * 
 * It might exist better way to implements those operations,
 * or they might already exist in other form JDT
 */

public class VMLauncherUtility {

	static public IVMInstall getVMInstall() {
		IVMInstallType[] vmTypes = JavaRuntime.getVMInstallTypes();
		for (int i = 0; i < vmTypes.length; i++) {
			IVMInstall[] vms = vmTypes[i].getVMInstalls();
			for (int j = 0; j < vms.length; j++) {
				if(vms[j].getId().equals(JonasLauncherPlugin.getDefault().getJonasJRE())) {
					return vms[j];	
				}
			}
		}
		return JavaRuntime.getDefaultVMInstall();		
	}


	static public void runVM(String label, String classToLaunch, String[] classpath, String[] bootClasspath, String[] vmArgs, String[] prgArgs, ISourceLocator sourceLocator, boolean debug, boolean showInDebugger) throws CoreException{
		runVM( label, classToLaunch, classpath, bootClasspath, vmArgs, prgArgs, sourceLocator, debug, showInDebugger, null );
	}


	static public void runVM( String	label,
								String	classToLaunch,
								String[] classpath,
								String[] bootClasspath,
								String[] vmArgs,
								String[] prgArgs,
								ISourceLocator sourceLocator,
								boolean debug,
								boolean showInDebugger,
								String	workingDir
							) throws CoreException
	{
		IVMInstall vmInstall = getVMInstall();
		String mode = "";
		if (debug)
			mode = ILaunchManager.DEBUG_MODE;
		else
			mode = ILaunchManager.RUN_MODE;

		IVMRunner vmRunner = vmInstall.getVMRunner(mode);
		
		ILaunchConfigurationType launchType = DebugPlugin.getDefault().getLaunchManager().getLaunchConfigurationType("org.eclipse.jdt.launching.localJavaApplication");	
		ILaunchConfigurationWorkingCopy config = launchType.newInstance(null, label);
		config.setAttribute(IDebugUIConstants.ATTR_PRIVATE, true);
		DebugUITools.setLaunchPerspective(launchType,"debug",IDebugUIConstants.PERSPECTIVE_DEFAULT);
		DebugUITools.setLaunchPerspective(launchType,"run",IDebugUIConstants.PERSPECTIVE_DEFAULT);
		config.setAttribute(IDebugUIConstants.ATTR_TARGET_DEBUG_PERSPECTIVE, IDebugUIConstants.PERSPECTIVE_DEFAULT);
		config.setAttribute(IDebugUIConstants.ATTR_TARGET_RUN_PERSPECTIVE, IDebugUIConstants.PERSPECTIVE_DEFAULT);
//		config.setAttribute(ILaunchConfiguration.ATTR_SOURCE_LOCATOR_ID, JavaUISourceLocator.ID_PROMPTING_JAVA_SOURCE_LOCATOR);		

		Launch launch = new Launch(config, mode, sourceLocator);

		config.doSave();

		if (vmRunner != null) {
			VMRunnerConfiguration vmConfig =
				new VMRunnerConfiguration(classToLaunch, classpath);
			vmConfig.setVMArguments(vmArgs);
			vmConfig.setProgramArguments(prgArgs);

			if( workingDir!=null ) { vmConfig.setWorkingDirectory( workingDir ); }

			if(bootClasspath.length == 0) {
				vmConfig.setBootClassPath(null); // use default bootclasspath	
			} else {
				vmConfig.setBootClassPath(bootClasspath);					
			}	


			vmRunner.run(vmConfig, launch, null);

		}

		// Show in debugger
		if(showInDebugger) {
			DebugPlugin.getDefault().getLaunchManager().addLaunch(launch);
		}

	}


	/**
	 * Register process to eclipse debugger
	 */
	static public void showInDebugger(ILaunch launch) throws CoreException {	
		DebugPlugin.getDefault().getLaunchManager().addLaunch(launch);
	}


}
