package com.bull.eclipse.jonas.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;
import java.util.zip.CRC32;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import com.bull.eclipse.jonas.ExtraResourcesEntries;


public class Zipper {

	private static final int BUFFER = 2048;

    private static final long EMPTY_CRC = new CRC32 ().getValue ();
    protected String emptyBehavior = "skip";
    private Vector filesets = new Vector ();
    
    protected Hashtable addedDirs = new Hashtable();
	private File outputFile = null;
	private File directory = null;
	private ExtraResourcesEntries extraList = null;
	private FileOutputStream fos = null;
	private ZipOutputStream zos = null;
	private String currentDirName;

	public Zipper(File outputFile, File directory) throws IOException {
		this.outputFile = outputFile;
		this.directory = directory;
		currentDirName = directory.getAbsolutePath();
	}
	
	public Zipper(File outputFile, File directory, ExtraResourcesEntries extraList) throws IOException {
		this.outputFile = outputFile;
		this.directory = directory;
		this.extraList = extraList;
		currentDirName = directory.getAbsolutePath();
	}
	
	public void zip() throws IOException {
		fos = new FileOutputStream(outputFile);
		zos = new ZipOutputStream(fos);
		zipDir(directory);
		zipExtraResources(extraList);
		ArrayList a ;
		zos.flush();
		zos.close();
		fos.close();		
	}


	static public void unzip(ZipEntry zipEntry,ZipFile zipFile,File output) throws IOException {
		copyInputStream(zipFile.getInputStream(zipEntry), new BufferedOutputStream(new FileOutputStream(output)));				
	}
	
	
	private void zipDir(File dir) throws IOException {
		if(!dir.getPath().equals(currentDirName)) {		
			String entryName = dir.getPath().substring(currentDirName.length()+1);
			entryName = entryName.replace('\\', '/');
	       	ZipEntry ze = new ZipEntry (entryName + "/");
	        if (dir != null && dir.exists()) {
	            ze.setTime(dir.lastModified());
	        } else {
	            ze.setTime(System.currentTimeMillis());
	        }
	        ze.setSize (0);
	        ze.setMethod (ZipEntry.STORED);
	        // This is faintly ridiculous:
	        ze.setCrc (EMPTY_CRC);
            zos.putNextEntry (ze);
		}
						
		if (dir.exists() && dir.isDirectory()) {
			File [] fileList = dir.listFiles();

			for (int i = 0; i < fileList.length; i++) {
				if (fileList[i].isDirectory() && this.acceptDir(fileList[i])) {
					zipDir(fileList[i]);
				}
				if (fileList[i].isFile() && this.acceptFile(fileList[i])) {				
					zipFile(fileList[i]);
				}
			}
		}		

	}


	private void zipExtraResources(ExtraResourcesEntries extraList) throws IOException {		
		if (extraList != null) {
			List resourcesList = extraList.getList();
			int i = 0;
			String tempFile = null;
			while (i < resourcesList.size()) {
				tempFile = new File ((String)resourcesList.get(i)).getName();
				zipFile(new File((String)resourcesList.get(i)) , "WEB-INF" + File.separator + "lib" + File.separator + tempFile);
				i++;
			}	
		}		
	}

	private void zipFile(File file) throws IOException {
		if(!file.equals(this.outputFile)) {
			BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file),BUFFER);
	
			String entryName = file.getPath().substring(currentDirName.length()+1);
			entryName = entryName.replace('\\', '/');
			ZipEntry fileEntry = new ZipEntry(entryName);
			zos.putNextEntry(fileEntry);
	
			byte[] data = new byte[BUFFER];
			int byteCount;
			while ((byteCount = bis.read(data, 0, BUFFER)) != -1) {
				zos.write(data, 0, byteCount);
			}
	
			bis.close();
		}
	}
	
	private void zipFile(File file, String zipEntry) throws IOException {
		if(!file.equals(this.outputFile)) {
			BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file),BUFFER);
	
			//String entryName = file.getPath().substring(currentDirName.length()+1);
			String entryName = zipEntry;
			entryName = entryName.replace('\\', '/');
			ZipEntry fileEntry = new ZipEntry(entryName);
			zos.putNextEntry(fileEntry);
	
			byte[] data = new byte[BUFFER];
			int byteCount;
			while ((byteCount = bis.read(data, 0, BUFFER)) != -1) {
				zos.write(data, 0, byteCount);
			}
	
			bis.close();
		}
	}
	
	protected boolean acceptDir(File dir) {
		return true;	
	}
	
	protected boolean acceptFile(File file) {
		return true;	
	}	
	
	
	private static final void copyInputStream(InputStream in, OutputStream out)
	  throws IOException
	{
		byte[] buffer = new byte[1024];
		int len;

		while((len = in.read(buffer)) >= 0)
		  out.write(buffer, 0, len);

		in.close();
		out.close();
	}

}
