/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Christophe Ney
 * --------------------------------------------------------------------------
 * $Id: EjbRefDesc.java,v 1.1 2003/10/27 14:46:32 benoitf Exp $
 * --------------------------------------------------------------------------
 */

package com.bull.eclipse.jonas.utils.xml;


/**
 * This class represents the description of an EjbRef object
 * @author Christophe Ney
 * @author Florent Benoit
 */
public class EjbRefDesc {

    /**
     * The ejb ref name.
     */
    private String ejbRefName = null;

    /**
     * The type of the ejb ref
     */
    private String ejbRefType = null;

    /**
     * The ejb link of the ejb local ref.
     */
    private String ejbLink = null;

    /**
     * The jndi name of the ejb local ref.
     */
    private String jndiName = null;


    /**
     * Construct a descriptor for an ejb-ref tag.
     * @param ejbRef the ejb ref resulting of the xml parsing.
     * @param jonasEjbRef the jonas ejb ref resulting of the xml parsing.
     * @throws DeploymentDescException when missing information for
     * creating the EjbRefDesc.
     */
//    public EjbRefDesc(EjbRef ejbRef, JonasEjbRef jonasEjbRef) throws DeploymentDescException {
//        ejbRefName = ejbRef.getEjbRefName();
//        ejbRefType = ejbRef.getEjbRefType();
//        ejbLink = (ejbRef.getEjbLink() != null) ? ejbRef.getEjbLink() : null;
//        jndiName = (jonasEjbRef != null) ? jonasEjbRef.getJndiName() : null;
//    }

    /**
     * Get the name of the ejb-ref 
     * @return String representation of the ejb-ref-name.
     */
    public String getEjbRefName() {
        return ejbRefName;
    }

    /**
     * Get the ejb-ref-type.
     * @return String representation of the ejb-ref-type.
     */
    public String getEjbRefType() {
        return ejbRefType;
    }

    /**
     * Get the ejb-link
     * @return String representation of the ejb-link
     */
    public String getEjbLink() {
        return ejbLink;
    }


    /**
     * Get the jndi name of the ejb-ref.
     * @return String representation of the JNDI name
     */
    public String getJndiName() {
        return jndiName;
    }

    /**
     * Set the jndi name of the ejb-ref.
     * @param jndiName representation of the JNDI name
     */
    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }


    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append("\ngetEjbRefName()=" + getEjbRefName());
        ret.append("\ngetEjbRefType()=" + getEjbRefType());
        ret.append("\ngetEjbLink()=" + getEjbLink());
        ret.append("\ngetJndiName()=" + getJndiName());
        return ret.toString();
    }

}
