/*
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: XMLSerializer.java,v 1.1 2004/01/21 15:30:19 sauthieg Exp $
 * --------------------------------------------------------------------------
*/

package com.bull.eclipse.jonas.utils.xml;

import java.io.IOException;
import java.io.OutputStream;

import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.LineSeparator;
import org.apache.xml.serialize.Method;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.Serializer;
import org.apache.xml.serialize.SerializerFactory;
import org.w3c.dom.Document;


/**
 * Serialize a given DOM Document into an outpoutStream.
 *
 * @author Guillaume Sauthier
 */
public class XMLSerializer {
    /**
     * Unique XMLSerilalizer instance
     */
    private static XMLSerializer instance = null;

    /**
     * Document to be formated and serialized
     */
    private Document doc;

    /**
     * XML Format
     */
    private OutputFormat format;

    /**
     * Serializer Factory
     */
    private SerializerFactory factory;

    /**
     * Creates a new XMLSerializer object.
     */
    private XMLSerializer() {
        // define the format for the xml document
        format = new OutputFormat();
        format.setLineSeparator(LineSeparator.Unix);
        format.setLineWidth(80);
        format.setIndent(4);

        // document serialization and writing
        factory = SerializerFactory.getSerializerFactory(Method.XML);

    }

    /**
     * Returns the unique XMLSerializer instance
     * configured for the given Document
     *
     * @param d Document to be formated
     *
     * @return configured XMLSerializer
     */
    public static XMLSerializer getInstance(Document d) {
        if(instance == null) {
            instance = new XMLSerializer();
        }

        instance.setDocument(d);

        return instance;
    }

    /**
     * Set the Current Doucment
     *
     * @param d current document
     */
    private void setDocument(Document d) {
        doc = d;
    }

    /**
     * Serialize the encapsulated Document into the OutputStream
     *
     * @param os output stream
     *
     * @throws IOException When serialization fails
     */
    public void serialize(OutputStream os) throws IOException {
        Serializer genericSerializer = factory.makeSerializer(os, format);
        DOMSerializer domSerializer = genericSerializer.asDOMSerializer();

        domSerializer.serialize(doc);
    }
}
