
package com.bull.eclipse.jonas.wizard;

import java.io.File;
import java.io.IOException;

import org.eclipse.ant.core.AntRunner;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.utils.FileUtil;


public class		CreateEARWizard	extends Wizard implements IWorkbenchWizard
{
	private CreateEARWizardPage		EarPage;
	private IStructuredSelection	selection;
	private IProject				proj;


	public			CreateEARWizard()
	{
		super();
		setDialogSettings( JavaPlugin.getDefault().getDialogSettings() );
		setWindowTitle(JonasLauncherPlugin.getResourceString("Create EAR"));
	}

	public void		addPages()
	{
		EarPage = new CreateEARWizardPage("EARWizardPage1"); 
		EarPage.setTitle("Create EAR"); 
		EarPage.setDescription("create an EAR file in a Jonas project");
		EarPage.setProj(proj);
		addPage(EarPage);
	}

	public void		init( IWorkbench workbench, IStructuredSelection selection )
	{
		this.selection = selection;
		Object	o = selection.getFirstElement();
		if( o instanceof JonasProject ) {
			proj = ((JonasProject)o).getProject();
		} else
		if( o instanceof IProject ) {
			proj = (IProject)o;
		} else
		if( o instanceof String ) {
			proj = ResourcesPlugin.getWorkspace().getRoot().getProject((String)selection.getFirstElement());
		}
	}

	public boolean	performFinish()
	{
		File	tempfilea = null;
		File	tempfileb = null;
		try {
			String	appxml = EarPage.getAppXml();
			tempfilea = saveStringToTempfile( "app.xml", appxml );
			String	bldxml = EarPage.getBldXml(tempfilea.toString());
			tempfileb = saveStringToTempfile( "bld.xml", bldxml );
			AntRunner	antrunner = new AntRunner();
			antrunner.setBuildFileLocation(tempfileb.toString());
			antrunner.run();
			try{ proj.refreshLocal( IResource.DEPTH_INFINITE, null ); }
			catch( CoreException e ) {}
		}
		catch( Exception e ) {
			// two exception types are possible: IOException, CoreException
			// no need to treat them differently
			MessageDialog.openInformation(null,"Error","exception occurred "+e);
			JonasLauncherPlugin.log("JOPE: exception in CreateEARWizard.performFinish "+e);
		}
		if( tempfilea!=null ) tempfilea.delete();
		if( tempfileb!=null ) tempfileb.delete();
		
		return true;
	}

	private File	saveStringToTempfile( String filename, String content )
										throws IOException
	{
		File	f = File.createTempFile( filename, "" );
		FileUtil.toTextFile( f, content );
		return f;
	}

}


