
package com.bull.eclipse.jonas.wizard;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;




public class		CreateEARWizardPage extends WizardPage
										implements ModifyListener, SelectionListener
{
	private Button			addClientJarButton;
	private Button			addEjbButton;
	private Button			addRarButton;
	private Button			addWarButton;
	private Text			clientText;
	private Text			clientUriText;
	private Text			contentsText;
	private Text			ctxRootText;
	private final String	ctxRoot_kwd = "_ctx";
	private Text			descriptionText;
	private Text			displaynameText;
	private ArrayList		EarContents = new ArrayList();
	private HashMap			EarContentsMisc = new HashMap();
	private Text			earText;
	private Text			ejbText;
	private Text			ejbUriText;
	private IProject		proj;
	private Text			rarText;
	private Text			rarUriText;
	private final String	type_kwd = "_type";
	private final String	type_client = "client";
	private final String	type_ejb = "ejb";
	private final String	type_rar = "rar";
	private final String	type_war = "war";
	private final String	uri_kwd = "_uri";
	private Text			warText;
	private Text			webUriText;


	public			CreateEARWizardPage( String pagename )
	{
		super(pagename);
	}

	public void		createControl( Composite parent )
	{
		Composite	composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));

		Composite earGroup = new Composite(composite,SWT.LEFT);
		GridLayout	layout = new GridLayout();
		layout.numColumns = 2;
		earGroup.setLayout(layout);
		earGroup.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );

		earText = addText(earGroup, "EAR file name");
		displaynameText = addText(earGroup,"display name:");
		descriptionText = addText(earGroup, "description:");
		contentsText = addText( earGroup, "contents:" );
		contentsText.setEditable( false );
		earText.addModifyListener(this);

		Group	ejbGroup = createGroup(composite,"EJB");
		ejbText = addText(ejbGroup,"JAR file name");
		ejbText.addModifyListener(this);
		ejbUriText = addText(ejbGroup,"ejb-uri:");
		addEjbButton = addADDbutton(ejbGroup);
		addEjbButton.addSelectionListener( new SelectionAdapter()
			{	public void	widgetSelected( SelectionEvent e )
				{ processAddEjbButton(); }
			}
		);

		Group	warGroup = createGroup(composite,"web app");
		warText = addText(warGroup,"WAR file name");
		warText.addModifyListener(this);
		webUriText = addText(warGroup,"web-uri:");
		ctxRootText = addText(warGroup,"context-root:");
		addWarButton = addADDbutton(warGroup);
		addWarButton.addSelectionListener( new SelectionAdapter()
			{	public void	widgetSelected( SelectionEvent e )
				{ processAddWarButton(); }
			}
		);

		Group	clientGroup = createGroup(composite,"client jar");
		clientText = addText(clientGroup,"JAR file name");
		clientText.addModifyListener(this);
		clientUriText = addText(clientGroup,"client-uri:");
		addClientJarButton = addADDbutton(clientGroup);
		addClientJarButton.addSelectionListener( new SelectionAdapter()
			{	public void	widgetSelected( SelectionEvent e )
				{ processAddClientJarButton(); }
			}
		);

		Group	rarGroup = createGroup(composite,"connector");
		rarText = addText(rarGroup,"RAR file name");
		rarText.addModifyListener(this);
		rarUriText = addText(rarGroup,"rar-uri:");
		addRarButton = addADDbutton(rarGroup);
		addRarButton.addSelectionListener( new SelectionAdapter()
			{	public void	widgetSelected( SelectionEvent e )
				{ processAddRarButton(); }
			}
		);

		setMessage(null);
		setControl(composite);
		setPageComplete( validatePage() );
	}

	private Group	createGroup( Composite parent, String title )
	{
		Group	g = new Group( parent, SWT.NONE );
		g.setText( title );
		//g.setLayout( new RowLayout() );
		g.setLayout( new GridLayout(2,false) );
		g.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		return g;
	}

	private Text	addText( Composite parent, String lblTitle )
	{
		new Label( parent, SWT.RIGHT ).setText( lblTitle );
		Text	t = new Text( parent, SWT.BORDER );
		GridData	data = new GridData(GridData.FILL_HORIZONTAL);
		t.setLayoutData(data);
		return t;
	}

	private Button	addADDbutton( Composite parent )
	{
		new Label(parent,SWT.NONE);
		Button	b = new Button(parent,SWT.NONE);
		b.setText("Add");
		b.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		return b;
	}

	private boolean	fileExists( String s )
	{
		File	f = proj.getFile(s).getLocation().toFile();
		boolean	b = f.exists();
		if( !b ) {
			MessageDialog.openInformation( null, "Error", "File does not exist in project: "+s );
		}
		return b;
	}

	private void	processAddClientJarButton()
	{
		String	s = clientText.getText().trim();
		if( s.length()>0 && fileExists(s) ) {
			addToContents(s,type_client,clientUriText);
			clientText.setText("");
			clientUriText.setText("");
			setPageComplete( validatePage() );
		}
	}

	private void	processAddEjbButton()
	{
		String	s = ejbText.getText().trim();
		if( s.length()>0 && fileExists(s) ) {
			addToContents(s,type_ejb,ejbUriText);
			ejbText.setText("");
			ejbUriText.setText("");
			setPageComplete( validatePage() );
		}
	}

	private void	processAddRarButton()
	{
		String	s = rarText.getText().trim();
		if( s.length()>0 && fileExists(s) ) {
			addToContents(s,type_rar,rarUriText);
			rarText.setText("");
			rarUriText.setText("");
			setPageComplete( validatePage() );
		}
	}

	private void	processAddWarButton()
	{
		String	s = warText.getText().trim();
		if( s.length()>0 && fileExists(s) ) {
			addToContents(s,type_war,webUriText);
			EarContentsMisc.put( s + ctxRoot_kwd, ctxRootText.getText().trim() );
			warText.setText("");
			webUriText.setText("");
			ctxRootText.setText("");	
			setPageComplete( validatePage() );
		}
	}

	private void	addToContents( String s2, String type, Text uri )
	{
		String	s1 = contentsText.getText();
		if( s1.length()>0 ) { s1 += ","; }
		contentsText.setText( s1 + s2 );
		EarContents.add( s2 );
		EarContentsMisc.put( s2 + type_kwd, type );
		String	s3 = uri.getText().trim();
		if( s3.length()==0 ) { s3 = basename(s2); }
		EarContentsMisc.put( s2 + uri_kwd, s3 );
	}

	private String	basename( String s )
	{
		int		i;
		String	s2 = s;
		if( (i=s2.lastIndexOf("/")) >= 0 ) { s2 = s2.substring(i+1); }
		return s2;
	}

	public void		modifyText( ModifyEvent e ) // ModifyListener implementation
	{
		setPageComplete( validatePage() );
	}

	public void		setProj( IProject p )
	{
		proj = p;
	}

	private boolean	validatePage()
	{
		return( earText != null
				&& earText.getText().trim().length() > 0
				&& !EarContents.isEmpty()
				&& ejbText.getText().trim().length()==0
				&& warText.getText().trim().length()==0
				&& clientText.getText().trim().length()==0
				&& rarText.getText().trim().length()==0
			);
	}

	public String	getAppXml()
	{
		return createAppXml();
	}

	public String	getBldXml( String appxmlLoc )
	{
		return createBldXml( appxmlLoc );
	}

	private String	createAppXml()	// create 'application.xml' contents
	{
		Date	now = new Date();
		String	appxml = "<?xml version='1.0' encoding='UTF-8'?>\n"
			+ "<!DOCTYPE application PUBLIC\n"
			+ "'-//Sun Microsystems, Inc.//DTD J2EE Application 1.3//EN'\n"
			+ "'http://java.sun.com/dtd/application_1_3.dtd'>\n"
			+ "<!-- generated by JOPE " + now + " -->\n"
			+ "<application>\n"
			+ "<display-name>"+displaynameText.getText().trim()+"</display-name>\n"
			+ "<description>"+descriptionText.getText().trim()+"</description>\n";
		for( int i=0; i<EarContents.size(); i++ ) {
			String	ar = (String)EarContents.get(i);
			appxml += "<module>\n";
			String	type = (String)EarContentsMisc.get( ar + type_kwd );
			String	uri = (String)EarContentsMisc.get( ar + uri_kwd );
			if( type.equals(type_war) ) {
				appxml += "<web>\n"
				+ "<web-uri>" + uri + "</web-uri>\n"
				+ "<context-root>" + EarContentsMisc.get(ar+ctxRoot_kwd) + "</context-root>\n"
				+ "</web>";
			}
			else if( type.equals(type_ejb) ) { appxml += "<ejb>" + uri + "</ejb>"; }
			else if( type.equals(type_client) ) { appxml += "<java>" + uri + "</java>"; }
			else /* rar */ { appxml += "<connector>" + uri + "</connector>"; }
			appxml += "</module>\n";
		}
		appxml += "</application>";
		return appxml;
	}

	private String	createBldXml( String appxml )
	{
		// create 'build.xml' containing 'ear' task
		Date	now = new Date();
		String	bldxml = "<project default='ear'>\n"
			+ "<!-- generated by JOPE " + now + " -->\n"
			+ "<target name='ear'>\n"
			+ "<ear destfile='"+ proj.getFile(earText.getText().trim()).getLocation() + "' "
			+ "appxml='" + appxml + "'>\n";
		for( int i=0; i<EarContents.size(); i++ ) {
			String	ar = (String)EarContents.get(i);
			String	uri = (String)EarContentsMisc.get( ar + uri_kwd );
			bldxml += "<zipfileset file='"
			+ proj.getFile(ar).getLocation()
			+ "' fullpath='" + uri + "'/>\n";
		}
		bldxml += "</ear>\n"
			+ "</target>\n"
			+ "</project>\n";
		return bldxml;
	}

	public void		widgetSelected( SelectionEvent e ) // SelectionListener
	{
		// This is useless because SelectionEvent.item IS NULL !!!
		// Must use a per-Button SelectionListener.
	}

	public void		widgetDefaultSelected( SelectionEvent e ) // SelectionListener
	{
		// this is not called for a Button
	}

}


