/*
 * Created on Sep 17, 2003
 *
 */
package com.bull.eclipse.jonas.wizard;

import java.io.File;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.viewsupport.IViewPartInputProvider;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPart;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.newbean.NewbeanRunner;

/**
 * @author steve krupanszky
 *
 * A wizard to create an EJB and related files for a Jonas project.
 * This is accomplished through a subordinate call to 'newbean' class,
 * which is provided by the JOnAS product.
 *
 */

public class	CreateEJBWizard  extends Wizard implements INewWizard 
{

	// fields listed in alphabetical order
	private CreateEJBWizardPage		EjbPage;
	private boolean					jonasproj = false;
	private String					jonasprojName = "";
	private boolean					jonasRootSet = false;
	private String					pkgName;
	private IProject				proj;
	private JonasProject			projNature;
	private IStructuredSelection	selection;
	private String					srcDir;
	private IWorkbench				workbench;


	public			CreateEJBWizard()
	{
		super();
		setDialogSettings( JavaPlugin.getDefault().getDialogSettings() );
		setWindowTitle(JonasLauncherPlugin.getResourceString("wizard.createEJB.windowTitle"));
	}

	/** @see Wizard
	*/
	public void		addPages()
	{
		if( jonasRootSet ) {
			EjbPage = new CreateEJBWizardPage("EJBWizardPage1"); 
			EjbPage.setTitle(JonasLauncherPlugin.getResourceString("wizard.createEJB.mainPage.title")); 
			EjbPage.setDescription(JonasLauncherPlugin.getResourceString("wizard.createEJB.mainPage.description"));
			if( jonasproj ) {
				EjbPage.setProjName(jonasprojName);
				EjbPage.setSrcDir(srcDir);
				EjbPage.setPkg(pkgName);
			}
			addPage(EjbPage);
		}
		else {
			EjbErrorWizardPage	errPage = new EjbErrorWizardPage("ErrorWizardPage1");
			errPage.setTitle(JonasLauncherPlugin.getResourceString("wizard.createEJB.errPage.title"));
			errPage.setErrorMessage(JonasLauncherPlugin.getResourceString("wizard.createEJB.errPage.text1"));
			addPage(errPage);
		}
	}

	private void	checkIfJonasRootSet()
	{
		String jonasRoot = JonasLauncherPlugin.getDefault().getJonasDir();
		String jonasBase = JonasLauncherPlugin.getDefault().getBaseDir();
		jonasRootSet = ( new File(jonasRoot).isDirectory() )
					&& (new File(jonasBase).isDirectory() );
	}

	private void	createFolderAndParents( IFolder f )
	{
		if( !f.getLocation().toFile().exists() ) {
			try{
				IContainer	p = f.getParent();
				if( p!=null && p instanceof IFolder ) {
					createFolderAndParents( (IFolder)p );
				}
				f.create( false,true,null );
			}
			catch( CoreException e ) {
				JonasLauncherPlugin.log("JOPE: exception creating folder: " + e );
			}
		}
	}

	/*
	 * returns the path relative to the project location (unlike
	 * getPath() which is relative to the workspace). used to get
	 * the complete name of a multi-component packageFragmentRoot.
	 */
	private String		getPathRelativeToProject( IJavaElement elem )
	{
		IJavaProject	proj = elem.getJavaProject();
		String	ps = proj.getPath().toString();
		String	es = elem.getPath().toString();
		if( es.startsWith(ps) ) {
			es = es.substring(ps.length());
			if( es.startsWith("/") ) { es = es.substring(1); }
		}
		return es;
	}
	private String		getPathRelativeToProject( IFolder f )
	{
		IProject	proj = f.getProject();
		String	ps = proj.getFullPath().toString();
		String	fs = f.getFullPath().toString();
		if( fs.startsWith(ps) ) {
			fs = fs.substring(ps.length());
			if( fs.startsWith("/") ) { fs = fs.substring(1); }
		}
		return fs;
	}

	/*
	 * returns an IFolder object for the package 'pkg' emanating from
	 * the source folder 'base'. 
	 */
	public static IFolder	getPkgFolder( IProject p, IFolder base, String pkg )
	{
		if( pkg==null || pkg.equals("") ) return base;
		IFolder		result = base;
		StringTokenizer		st = new StringTokenizer(pkg,".");
		while( st.hasMoreTokens() ) {
			String	tok = st.nextToken();
			if( result==null ) { result = p.getFolder(tok); }
			else { result = result.getFolder(tok); }
		}
		return result;
	}

	void			info( String msg )
	{
		MessageDialog.openInformation(
			/*workbench.getActiveWorkbenchWindow().*/getShell(),
			"Info",
			msg );
	}

	/** @see INewWizard
	*/
	public void		init( IWorkbench wb, IStructuredSelection sel )
	{
		workbench = wb;
		selection = sel;

		checkIfJonasRootSet();

		//showDebugInfo( sel );
		// determine if selection represents a jonas project
		IJavaElement	jelem = getInitialJavaElement( sel );
		if( jelem!=null ) {
			IJavaProject	jproj = jelem.getJavaProject();
			if( jproj!=null ) {
				IPackageFragment	frag;
				//THIS DOESN'T WORK: frag = (IPackageFragment)jelem.getAdapter(IPackageFragment.class);
				frag = (IPackageFragment)jelem.getAncestor(IJavaElement.PACKAGE_FRAGMENT);
				if( frag!=null ) {
					pkgName = frag.getElementName();
				}
				IPackageFragmentRoot	fragroot;
				fragroot = (IPackageFragmentRoot)jelem.getAncestor(IJavaElement.PACKAGE_FRAGMENT_ROOT);
				if( fragroot!=null ) {
					srcDir = getPathRelativeToProject(fragroot);
				} else {
					// it's not a fragroot, see if it's just a folder
					Object selectedElement = selection.getFirstElement();
					if( selectedElement instanceof IFolder ) {
						srcDir = getPathRelativeToProject((IFolder)selectedElement);
					}
				}
				
				jonasprojName = jproj.getElementName();
				proj = jproj.getProject();
				try {
					jonasproj = proj.hasNature(JonasLauncherPlugin.NATURE_ID);
					projNature = (JonasProject)proj.getNature(JonasLauncherPlugin.NATURE_ID);
				}
				catch( CoreException e ) {}
			}
		}
	}

	/** @see Wizard
	*/
	public boolean		performFinish()
	{
		// get the project/srcdir/pkg names from the wizard page
		String		pkg = EjbPage.getPkgName();
		srcDir = projNature.getSrcEJBDir();
		if (srcDir.trim().equals("")) {
			srcDir = "components";
			projNature.setSrcEJBDir("components");
			projNature.saveProperties();
			try {
				projNature.createSrcEJBFolder();
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		jonasprojName = EjbPage.getProjName();
		// get the corresponding JonasProject object
		proj = ResourcesPlugin.getWorkspace().getRoot().getProject(jonasprojName);
		projNature = JonasProject.create(proj);
		// make sure that all directories in the path exist
		IFolder		pkgfolder = null;
		if( srcDir.length()>0 ) { pkgfolder = proj.getFolder(srcDir); }
		pkgfolder = getPkgFolder( proj, pkgfolder, pkg );
		if( !pkgfolder.getLocation().toFile().exists() ) {
			// (this can't be the best way to do this)
			createFolderAndParents( pkgfolder );
		}
// now call the Jonas 'newbean' class
		if (!EjbPage.getEJBType().equals("Deploy Unit")) {
			NewbeanRunner runner = new NewbeanRunner(projNature);
			runner.setNature(projNature);
			runner.setPkgFolder(pkgfolder); // tell it where to put java files
			runner.RunNewbean(EjbPage);
		} else {
			// now to refresh project so it picks up the new files
			try{
				projNature.setPackageProjectFolder(EjbPage.getPkgName());
				projNature.setJarEjbName(EjbPage.getJarName());
				projNature.getProject().refreshLocal( IResource.DEPTH_INFINITE, null );			 
			}
			catch( CoreException e ) {}

		}
		return true;
	}

	private void		showDebugInfo( IStructuredSelection sel )
	{
		boolean	hasJonasNature = false;
		String	jelemCls = "null";
		String	jelemName = "n/a";
		String	jelemProj = "n/a";
		Object	obj = sel.getFirstElement();
		String	selDescrip = "empty";

		if( obj!=null ) {
			selDescrip = obj.getClass().toString();
			IJavaElement	jelem = getInitialJavaElement( sel );
			if( jelem!=null ) {
				jelemCls = jelem.getClass().getName();
				jelemName = jelem.getElementName();
				IJavaProject	jproj = jelem.getJavaProject();
				if( jproj!=null ) {
					jelemProj = jproj.getElementName();
					try{ hasJonasNature = jproj.getProject().hasNature(JonasLauncherPlugin.NATURE_ID); }
					catch( CoreException e ) {}
				}
			}
		}
		info(
			"selection is: " + selDescrip + "\n" +
			"corr java element is: " + jelemName +
			" of type " + jelemCls + "\n" +
			"in project: " + jelemProj +
			(hasJonasNature?" (a Jonas project)":"")
		);
	}


	/**
	 * Utility method to inspect a selection to find a Java element.
	 * from org.eclipse.jdt.ui.wizards
	 *
	 * @param selection the selection to be inspected
	 * @return a Java element to be used as the initial selection, or <code>null</code>
	 * if no Java element exists in the given selection
	 */
	protected IJavaElement	getInitialJavaElement( IStructuredSelection selection )
	{
		IJavaElement jelem= null;
		if( selection!=null && !selection.isEmpty() ) {
			Object selectedElement = selection.getFirstElement();
			if( selectedElement instanceof IAdaptable ) {
				IAdaptable adaptable = (IAdaptable)selectedElement;			
				
				jelem = (IJavaElement)adaptable.getAdapter(IJavaElement.class);
				if( jelem==null ) {
					IResource resource = (IResource)adaptable.getAdapter(IResource.class);
					if( resource!=null && resource.getType()!=IResource.ROOT ) {
						while( jelem==null && resource.getType()!=IResource.PROJECT ) {
							resource = resource.getParent();
							jelem = (IJavaElement)resource.getAdapter(IJavaElement.class);
						}
						if( jelem==null ) {
							jelem = JavaCore.create(resource); // java project
						}
					}
				}
			}
		}
		if( jelem==null ) {
			IWorkbenchPart part = JavaPlugin.getActivePage().getActivePart();
			if( part instanceof IEditorPart  ) {
				part = JavaPlugin.getActivePage().getActiveEditor();
			}
			
			if( part instanceof IViewPartInputProvider ) {
				Object elem = ((IViewPartInputProvider)part).getViewPartInput();
				if( elem instanceof IJavaElement ) {
					jelem = (IJavaElement)elem;
				}
			}
		}
		if( jelem==null || jelem.getElementType() == IJavaElement.JAVA_MODEL ) {
			try {
				IJavaProject[] projects = JavaCore.create(ResourcesPlugin.getWorkspace().getRoot()).getJavaProjects();
				if( projects.length==1 ) {
					jelem = projects[0];
				}
			} catch( Exception e ) {}
		}
		return jelem;
	}


}
