package com.bull.eclipse.jonas.wizard;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
import org.eclipse.jdt.internal.ui.wizards.NewWizardMessages;
import org.eclipse.jdt.ui.wizards.NewJavaProjectWizardPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.actions.WorkspaceModifyDelegatingOperation;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasPluginResources;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;
import com.bull.eclipse.newwebapp.NewWebappsRunner;
//import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
//import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;

public class JonasProjectCreationWizard extends Wizard implements IExecutableExtension,INewWizard, JonasPluginResources  {

	public static final String NEW_PROJECT_WIZARD_ID= "org.eclipse.jdt.ui.wizards.NewProjectCreationWizard"; //$NON-NLS-1$
		
	private NewJonasProjectWizardPage fJonasPage;
	private NewJavaProjectWizardPage fJavaPage;
	private WizardNewProjectCreationPage fMainPage;
	private IConfigurationElement fConfigElement;
	private ISelection selection;
	
	public JonasProjectCreationWizard() {
		super();
		setNeedsProgressMonitor(true);
		ImageDescriptor banner = this.getBannerImg();
		if(banner != null) 		
			setDefaultPageImageDescriptor(banner);
		setDialogSettings(JavaPlugin.getDefault().getDialogSettings());
		setWindowTitle(WIZARD_PROJECT_TITLE);
	}

	private ImageDescriptor getBannerImg() {
		try {
			URL prefix = new URL(JonasLauncherPlugin.getDefault().getDescriptor().getInstallURL(), "icons/");
			return ImageDescriptor.createFromURL(new URL(prefix, "newjprj_wiz.gif"));
		} catch (MalformedURLException e) {
			return null;
		}
	}
	
	/*
	 * @see Wizard#addPages
	 */	
	public void addPages() {		
		fMainPage= new WizardNewProjectCreationPage("JOnASNewProjectPage"); 
		fMainPage.setTitle(WIZARD_PROJECT_MAINPAGE_TITLE); 
		fMainPage.setDescription(WIZARD_PROJECT_MAINPAGE_DESCRIPTION); 
		addPage(fMainPage);

		fJonasPage = new NewJonasProjectWizardPage(selection,"NewJonasProjectPage");
		fJonasPage.setTitle(WIZARD_PROJECT_JONASPAGE_TITLE); 
		fJonasPage.setDescription(WIZARD_PROJECT_JONASPAGE_DESCRIPTION);
		addPage(fJonasPage);
		
		IWorkspaceRoot root= JavaPlugin.getWorkspace().getRoot();
		fJavaPage= new NewJavaProjectWizardPage(root, fMainPage);
//		addPage(fJavaPage);
	}		
	

	/*
	 * @see Wizard#performFinish
	 */		
	public boolean performFinish() {
		IRunnableWithProgress op= new WorkspaceModifyDelegatingOperation(fJavaPage.getRunnable());;
		try {
			getContainer().run(false, true, op);
			JonasProject.addJonasNature(fJavaPage.getNewJavaProject());
			JonasProject jonasPrj = JonasProject.create(fJavaPage.getNewJavaProject());
			FileUtil.copyFile(new File(TemplateDirUtil.getTemplateDir().concat("/build.properties")),new File(jonasPrj.getProject().getLocation().append("/build.properties").toOSString()));
			if (fJonasPage.isCreateWebApp()) {			
				jonasPrj.setWebContext(fJonasPage.getWebcontext());			
				jonasPrj.setSrcDir(fJonasPage.getSrcpath());
				jonasPrj.setWebdirName(fJonasPage.getWebname());
				jonasPrj.setWarLocation(fJonasPage.getWarlocation());
				jonasPrj.saveProperties();
				jonasPrj.setCreateSamples( fJonasPage.isSample() );
				jonasPrj.fullConfiguration();
				NewWebappsRunner runner = new NewWebappsRunner(jonasPrj);				
				runner.RunNewwebapps();				
			} else {
				jonasPrj.addJonasJarToProjectClasspath();
				jonasPrj.clearDefaultSourceEntries();
				jonasPrj.setClassesAsOutputFolder();							
			}
			if (fJonasPage.isEJBPrj()) {
				jonasPrj.setSrcEJBDir(fJonasPage.getSrcEJBpath());
				jonasPrj.saveProperties();
				jonasPrj.createSrcEJBFolder();
			}						
		} catch (InvocationTargetException e) {
			String title= NewWizardMessages.getString("NewProjectCreationWizard.op_error.title"); //$NON-NLS-1$
			String message= NewWizardMessages.getString("NewProjectCreationWizard.op_error.message");			 //$NON-NLS-1$
			ExceptionHandler.handle(e, getShell(), title, message);
			return false;
		} catch  (InterruptedException e) {
			return false;
		} catch (CoreException e) {
			return false;	
		} catch (IOException e) {
			return false;	
		}
		
		BasicNewProjectResourceWizard.updatePerspective(fConfigElement);
//		selectAndReveal(fJavaPage.getNewJavaProject().getProject());
		return true;
	}

			
	/*
	 * Stores the configuration element for the wizard.  The config element will be used
	 * in <code>performFinish</code> to set the result perspective.
	 */
	public void setInitializationData(IConfigurationElement cfig, String propertyName, Object data) {
		fConfigElement= cfig;
	}
	
	

	/*
	 * @see IWizard#getNextPage(IWizardPage)
	 */
	public IWizardPage getNextPage(IWizardPage page) {
		// initialize Jonas Wizard page webpath field
		// Default value is / + projectName

		if (page instanceof WizardNewProjectCreationPage) {
			if(!fJonasPage.wasDisplayedOnce()) {
				fJonasPage.setWebcontext(fMainPage.getProjectName());
				fJonasPage.setWebname("webApplication");					
				fJonasPage.setSrcpath("source");									
			}			
		}
		return super.getNextPage(page);
	}
	
	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}

}