package com.bull.eclipse.jonas.wizard;

/*
 * (c) Copyright Bull SA 2003.
 * All Rights Reserved.
 */


import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.bull.eclipse.jonas.JonasPluginResources;


public class NewJonasProjectWizardPage extends WizardPage implements JonasPluginResources {

	private Button createWebApp;
	private Button createEJB;
	private Text webcontextText;
	private Text webnameText;
	private Text websrcText;
	private String warLocation;
	private Button createSample;
	private ISelection selection;
	private Button ejbInProject;

	
	private Text srcDirEJB;

	//private Text rootDirText;
	private JonasProjectCreationWizard wizard;

	// See JonasProjectCreationWizard.getNextPage
	private boolean displayedOnce = false;

	private static final int TEXT_FIELD_WIDTH = 200;

	/**
	 * Creates a new project creation wizard page.
	 *
	 * @param pageName the name of this page
	 */
	public NewJonasProjectWizardPage(ISelection selection, String pageName) {
		super(pageName);
		this.selection = selection;
	}

	/*
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.NULL);

		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		createWebGroup(composite);
		createEJBGroup(composite);


		setErrorMessage(null);
		setMessage(null);
		setControl(composite);
	}


	public void createWebGroup(Composite parent) {

		Composite webGroup = new Composite(parent,SWT.LEFT);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		webGroup.setLayout(layout);
		webGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// project location entry field
		createWebApp = new Button(webGroup, SWT.CHECK | SWT.LEFT);
		createWebApp.setText(WIZARD_PROJECT_CREATEWEBAPP_LABEL);
		createWebApp.setEnabled(true);
		createWebApp.setSelection(true);
		createWebApp.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent ev) {
						if (createWebApp.getSelection()) {
							webcontextText.setEnabled(true);
							webnameText.setEnabled(true);
							websrcText.setEnabled(true);
						} else {							
							webcontextText.setEnabled(false);
							webnameText.setEnabled(false);
							websrcText.setEnabled(false);												
						}
					}});


		new Label(webGroup, SWT.NULL);

		// location label
		Label webpathLabel = new Label(webGroup,SWT.LEFT);
		webpathLabel.setText(WIZARD_PROJECT_WEBPATH_LABEL);
		webpathLabel.setEnabled(true);

		// project location entry field
		webcontextText = new Text(webGroup, SWT.BORDER);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
		webcontextText.setLayoutData(data);
		webcontextText.setText(""); // see JonasProjectCreationWizard.nextPage


		//location label
		Label webnameLabel = new Label(webGroup,SWT.LEFT);
		webnameLabel.setText(WIZARD_PROJECT_WEBNAME_LABEL);

		// project location entry field
		webnameText = new Text(webGroup, SWT.BORDER);
		GridData dataname = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
		webnameText.setLayoutData(dataname);
		webnameText.setText(""); // see JonasProjectCreationWizard.nextPage


		// location label
		Label websrcLabel = new Label(webGroup,SWT.LEFT);
		websrcLabel.setText(WIZARD_PROJECT_WEBSRC_LABEL);
		websrcLabel.setEnabled(true);

		// project location entry field
		websrcText = new Text(webGroup, SWT.BORDER);
		GridData datasrc = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = TEXT_FIELD_WIDTH;
		websrcText.setLayoutData(datasrc);
		websrcText.setText(""); // see JonasProjectCreationWizard.nextPage

		createSample = new Button( webGroup, SWT.CHECK | SWT.LEFT );
		createSample.setText(WIZARD_PROJECT_CREATESRC_LABEL);
		createSample.setEnabled(true);
		createSample.setSelection(false);


	}

	public void createEJBGroup(Composite parent) {
		Composite ejbGroup = new Composite(parent,SWT.LEFT);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		ejbGroup.setLayout(layout);
		ejbGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		ejbInProject = new Button( ejbGroup, SWT.CHECK | SWT.LEFT );
		ejbInProject.setText(WIZARD_PROJECT_EJBINPROJECT_LABEL);
		ejbInProject.setEnabled(true);
		ejbInProject.setSelection(false);
		ejbInProject.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent ev) {
						if (ejbInProject.getSelection()) {
							srcDirEJB.setEnabled(true);
						} else {							
							srcDirEJB.setEnabled(false);
						}
					}});

		new Label(ejbGroup, SWT.NULL);

		// location label
		Label ejbsrcLabel = new Label(ejbGroup,SWT.LEFT);
		ejbsrcLabel.setText(WIZARD_PROJECT_EJBSRC_LABEL);

		// project location entry field
		srcDirEJB = new Text(ejbGroup, SWT.BORDER);
		GridData datasrc = new GridData(GridData.FILL_HORIZONTAL);
		datasrc.widthHint = TEXT_FIELD_WIDTH;
		srcDirEJB.setLayoutData(datasrc);
		srcDirEJB.setText("components"); // see JonasProjectCreationWizard.nextPage
		srcDirEJB.setEnabled(false);

	}


	public boolean isWebApp() {
			return createWebApp.getSelection();
	}

	public String getWebcontext() {
		return webcontextText.getText();
	}

	public String getSrcpath() {
			return websrcText.getText();
	}

	public String getWebname() {
			return webnameText.getText();
	}

	public String getWarlocation() {
			return warLocation;
	}


	public void setWebcontext(String path) {
		warLocation = path + ".war";
		webcontextText.setText(path);
	}

	public void setSrcpath(String path) {
		websrcText.setText(path);
	}

	public void setWebname(String name) {
		webnameText.setText(name);
	}

	public boolean isCreateWebApp() {
		return createWebApp.getSelection();
	}

	public boolean isEJBApp() {
		return createEJB.getSelection();
	}

	public String getSrcEJBpath() {
		return srcDirEJB.getText();
	}


	public boolean isSample() {

			return createSample.getSelection();
	}

	public boolean isEJBPrj() {
		return ejbInProject.getSelection();
	}



	/*
	 * @see IWizardPage#canFlipToNextPage()
	 */
	public boolean canFlipToNextPage() {
		displayedOnce = true;
		return super.canFlipToNextPage();
	}


	/**
	 * Gets the wasDisplayedOnce.
	 * @return Returns a boolean
	 */
	public boolean wasDisplayedOnce() {
		return displayedOnce;
	}


}
