/*
 * Created on Sep 22, 2003
 *
 */
package com.bull.eclipse.newear;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;

import com.bull.eclipse.jonas.JonasLauncherPlugin;
import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.utils.AntUtils;
import com.bull.eclipse.jonas.utils.EjbManagement;
import com.bull.eclipse.jonas.utils.FileUtil;
import com.bull.eclipse.jonas.utils.RarManagement;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;

/**
 * Interface with the JOnAS 'newbean' functionality.
 * @author steve krupanszky
 *
 */
public class NewEarRunner
{
	//private JonasProject	jonasNature;
	private IFolder			pkgfolder;	// the folder in which the java files will go
	private JonasProject		project;
	private Vector ejbListe;
	private Vector webListe;
	private Vector rarListe;
	private VelocityEngine vEngine = null;
	private VelocityContext vContext = null;
	
	private static final int EXIT_SUCCESS = 0;
	private static final int EXIT_FAILURE = 1;


	public			NewEarRunner( JonasProject proj, Vector ejbListe, Vector webListe, Vector rarListe)
	{
		project = proj;
		this.ejbListe = ejbListe;
		this.webListe = webListe;
		this.rarListe = rarListe;
	}

	/**
	 * Copy file into web directory, if it doesn't already exist there.
	 */
	private void	copyFileToEarDirectory( File f )
	{
		IFile	xmlfile = project.getProject().getFile( f.getName() );
		try{ FileUtil.copy( f, xmlfile.getLocation().toFile() ); }
		catch( IOException e ) {
			JonasLauncherPlugin.log(
				"JOPE: exception copying file " + f.getName()
				+ " to project " + project.getProject().getName() + ": " + e );
		}
	}

	private boolean		myDelete( File f )
	{
		boolean		b = f.delete();
		int			retryCount = 5;
		while( !b && retryCount-->0 ) {
			try{ java.lang.Thread.sleep(1000); } catch(Exception e) {}
			b = f.delete();
		}
		return b;
	}

	public void		RunNewear()
	{
		vContext = new VelocityContext();

		String jonasRoot = JonasLauncherPlugin.getDefault().getJonasDir();
		String earName = project.getProject().getName() + ".ear";
		
		vContext.put("earName",earName);
		
		if (!webListe.isEmpty()) {
			String	warContext = project.getWebContext();
			vContext.put("warContext",warContext);
			vContext.put("webDir",project.getWebdirName());
		}
		if (!ejbListe.isEmpty()) {
			vContext.put("ejbListe",ejbListe);
			vContext.put("pkgListe",EjbManagement.getPackageNames(ejbListe,project));
		}

		if (!rarListe.isEmpty()) {
			vContext.put("rarListe",rarListe);
		}
				
		vContext.put("outputEAR",project.getProject().getLocation().toOSString());
		vContext.put("srcEjbDir", project.getProject().getLocation().toOSString() + File.separator + project.getSrcEJBDir());
		
		File outputDir = new File(project.getProject().getLocation().toOSString());
		

		// Generates webapps files
		try {
			vEngine = new VelocityEngine();
			vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
			vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
			vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
							TemplateDirUtil.getTemplateDir().concat("/newear"));
			try {
				vEngine.init();
			} catch (Exception e) {
				fatalError("unable to initilise Velocity engine (" + e + ")");
			}
			generate("build.vm", outputDir + "/build.xml");            			
			generate("application.vm", outputDir + "/application.xml");
			System.out.println("Your build.xml file for ear have been created. You can now customize them.");
		} catch (Exception e) {        	
			error(e.toString());
		}
		// now to refresh project so it picks up the new files
		try{ project.getProject().refreshLocal( IResource.DEPTH_INFINITE, null ); }
		catch( CoreException e ) {}

		if (!rarListe.isEmpty()) {
			Enumeration ele = rarListe.elements();
			String pathRars = project.getProject().getLocation().append("/resourceAdapter").toOSString();			
			while (ele.hasMoreElements()) {
				try {
					RarManagement.buildRarFile(pathRars + "/" + ele.nextElement(),project.getProject());
				} catch (IOException ioe) {
					JonasLauncherPlugin.log("Problems when creating the rars files");
				}
			}					
		}
		
		AntUtils.runAnt(project.getProject().getFullPath().append("/build.xml").toOSString());
	}

	public void			setPkgFolder( IFolder f )
	{
		pkgfolder = f;
	}

	/**
	 * Create a new array : [ s1, s2, s3 ]
	 * (TODO move this method to StringUtil)
	 */
	public static String[] concat( String[] s1, String s2, String s3 )
	{
		String[] full = new String[ s1.length + 2 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-2 ] = s2;
		full[ full.length-1 ] = s3;
		return full;	
	}

	/**
	 * Create a new array : [ s1, s2 ]
	 * (TODO move this method to StringUtil)
	 */
	public static String[] concat( String[] s1, String s2 )
	{
		String[] full = new String[ s1.length + 1 ];
		System.arraycopy( s1, 0, full, 0, s1.length );
		full[ full.length-1 ] = s2;
		return full;	
	}

	static void fatalError(String errMsg) {
		System.err.println("NewBean fatal error: " + errMsg);
		System.exit(EXIT_FAILURE);
	}

	/**
	  * Generates a file from the specified template.
	  * @param templateFileName the name of the template file
	  * @param targetFileName the name of the generated file
	  */
	 private void generate(String templateFileName,
						   String targetFileName) throws Exception, IOException, ResourceNotFoundException, ParseErrorException, MethodInvocationException {
		 FileWriter fileWriter = null;
		 fileWriter = new FileWriter(targetFileName);
		 vEngine.mergeTemplate(templateFileName, vContext, fileWriter);
		 fileWriter.close();
	 }

	/**
	 * Display the specified error message.
	 * @param errMsg the error message to display
	 */
	static void error(String errMsg) {
		System.err.println("NewBean error: " + errMsg);
	}


}
