package com.bull.eclipse.persitence;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Vector;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.eclipse.jdt.ui.wizards.NewTypeWizardPage;

import com.bull.eclipse.jonas.JonasProject;
import com.bull.eclipse.jonas.actions.hibernate.CompositePKValue;
import com.bull.eclipse.jonas.actions.hibernate.GlobalHBMValue;
import com.bull.eclipse.jonas.actions.hibernate.PersistenceServicesValue;
import com.bull.eclipse.jonas.utils.TemplateDirUtil;



/**
 * This class is a &ldquo;bean generator&rdquo;. It uses Velocity and
 * a set of template files to generate some files that can serve as a
 * startup point for the developpement of a new bean.
 */
public class HibernateServicePersistenceGenerate {

    private static final int EXIT_SUCCESS = 0;
    private static final int EXIT_FAILURE = 1;

    private VelocityEngine  vEngine  = null;
    private VelocityContext vContext = null;
	private JonasProject prj = null;


    public HibernateServicePersistenceGenerate(JonasProject prj) {
    	this.prj = prj;
    }


    /**
     * Generates files for this new bean.
     */
    public void generateServicePersistence(PersistenceServicesValue psv) {

    	try {
			vEngine = new VelocityEngine();
			vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
			vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
			vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
					TemplateDirUtil.getTemplateDir().concat("/hibernate"));
			try {
				vEngine.init();
			} catch (Exception e) {
				fatalError("unable to initilise Velocity engine (" + e + ")");
			}

			// Creates a Velocity context and populates it by walking
			// through the parameter set
			vContext = new VelocityContext();
			vContext.put("persistenceServicesValue", psv);
			
			File persistenceServiceDir = new File(prj.getProject().getLocation().append("persistence").append(psv.getPersistencePackage().replace('.',File.separatorChar)).append("persistenceservices").toOSString());
			if (!persistenceServiceDir.exists()) 
				persistenceServiceDir.mkdirs();
			File utilPersistenceServiceDir = new File(prj.getProject().getLocation().append("persistence").append(psv.getPersistencePackage().replace('.',File.separatorChar)).append("persistence").append("util").toOSString());
			if (!utilPersistenceServiceDir.exists()) 
				utilPersistenceServiceDir.mkdirs();			
			File exceptionPersistenceServiceDir = new File(prj.getProject().getLocation().append("persistence").append(psv.getPersistencePackage().replace('.',File.separatorChar)).append("persistence").append("exception").toOSString());
			if (!exceptionPersistenceServiceDir.exists()) 
				exceptionPersistenceServiceDir.mkdirs();			

			generate("HibernatePersistenceServices.vm", new File(persistenceServiceDir, psv.getPersistenceName() + "Impl.java"));
			generate("InterfacePersistence.vm", new File(persistenceServiceDir, psv.getPersistenceName() + ".java"));
			generate("PersistenceUtil.vm", new File(persistenceServiceDir, psv.getPersistenceName() + "Util.java"));
			generate("HibernateUtil.vm", new File(utilPersistenceServiceDir, "HibernateUtil.java"));
			generate("PersistenceException.vm", new File(exceptionPersistenceServiceDir, "PersistenceException.java"));
        } catch (Exception e) {
            error(e.toString());
        }
	
    }


    public void generateCompositePK(ArrayList cpk, String packageName, String className) {

    	try {
			vEngine = new VelocityEngine();
			vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
			vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
			vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH,
					TemplateDirUtil.getTemplateDir().concat("/hibernate"));
			try {
				vEngine.init();
			} catch (Exception e) {
				fatalError("unable to initilise Velocity engine (" + e + ")");
			}

			// Creates a Velocity context and populates it by walking
			// through the parameter set
			vContext = new VelocityContext();
			vContext.put("listeCompositePK", cpk);
			vContext.put("packageName", packageName);
			vContext.put("className", className);
			
			File tempDir = new File(prj.getProject().getLocation().append("temp").toOSString());
			if (!tempDir.exists()) 
				tempDir.mkdirs();
			generate("composite_id.vm", new File(tempDir, "composite_id.xml"));
        } catch (Exception e) {
            error(e.toString());
        }
	
    }

    /**
     * Generates a file from the specified template.
     * @param templateFileName the name of the template file
     * @param targetFileName the name of the generated file
     */
    private void generate(String templateFileName,
                          File targetFileName) throws Exception, IOException, ResourceNotFoundException, ParseErrorException, MethodInvocationException {
        FileWriter fileWriter = null;
        fileWriter = new FileWriter(targetFileName);
        vEngine.mergeTemplate(templateFileName, vContext, fileWriter);
        fileWriter.close();
    }


    /**
     * Display the specified error message.
     * @param errMsg the error message to display
     */
    static void error(String errMsg) {
        System.err.println("NewBean error: " + errMsg);
    }


    /**
     * Display the specified error message and exits with an
     * EXIT_FAILURE status.
     * @param errMsg the error message to display
     */
    static void fatalError(String errMsg) {
        System.err.println("NewBean fatal error: " + errMsg);
        System.exit(EXIT_FAILURE);
    }

}
